@@ -26,10 +26,20 @@
 #if PY_MAJOR_VERSION >= 3
     #define PyString_AsString PyUnicode_AsUTF8
     #define PyString_Check PyUnicode_Check
+    #define PyString_FromFormat PyUnicode_FromFormat
     #define PyString_FromString PyUnicode_FromString
     #define PyString_InternFromString PyUnicode_InternFromString
     #define PyInt_Type PyLong_Type
 #endif
 
+
+#ifndef Py_MIN
+#define Py_MIN(x, y) (((x) > (y)) ? (y) : (x))
+#endif
+
+#ifndef Py_MAX
+#define Py_MAX(x, y) (((x) < (y)) ? (y) : (x))
+#endif
+
 #endif /* NUMBA_PY_MODULE_H_ */
 
