@@ -54,9 +54,10 @@ class NVVM(object):
         # nvvmResult nvvmDestroyProgram(nvvmProgram *cu)
         'nvvmDestroyProgram': (nvvm_result, POINTER(nvvm_program)),
 
-        # nvvmResult nvvmAddModuleToProgram(nvvmProgram cu, const char *buffer, size_t size)
+        # nvvmResult nvvmAddModuleToProgram(nvvmProgram cu, const char *buffer,
+        #                                   size_t size, const char *name)
         'nvvmAddModuleToProgram': (
-            nvvm_result, nvvm_program, c_char_p, c_size_t),
+            nvvm_result, nvvm_program, c_char_p, c_size_t, c_char_p),
 
         # nvvmResult nvvmCompileProgram(nvvmProgram cu, int numOptions,
         #                          const char **options)
@@ -138,7 +139,7 @@ class CompilationUnit(object):
            representation (LLVM3.0) or in the text representation.
         """
         err = self.driver.nvvmAddModuleToProgram(self._handle, buffer,
-                                                 len(buffer))
+                                                 len(buffer), None)
         self.driver.check_error(err, 'Failed to add module')
 
     def compile(self, **options):
@@ -330,17 +331,90 @@ define internal i32 @___numba_cas_hack(i32* %ptr, i32 %cmp, i32 %val) alwaysinli
 }
 """
 
+# Translation of code from CUDA Programming Guide v6.5, section B.12
+ir_numba_atomic_double_add = """
+define internal double @___numba_atomic_double_add(double* %ptr, double %val) alwaysinline {
+entry:
+    %iptr = bitcast double* %ptr to i64*
+    %old2 = load volatile i64* %iptr
+    br label %attempt
+
+attempt:
+    %old = phi i64 [ %old2, %entry ], [ %cas, %attempt ]
+    %dold = bitcast i64 %old to double
+    %dnew = fadd double %dold, %val
+    %new = bitcast double %dnew to i64
+    %cas = cmpxchg volatile i64* %iptr, i64 %old, i64 %new monotonic
+    %repeat = icmp ne i64 %cas, %old
+    br i1 %repeat, label %attempt, label %done
+
+done:
+    %result = bitcast i64 %old to double
+    ret double %result
+}
+"""
+
+ir_numba_atomic_double_max = """
+define internal double @___numba_atomic_double_max(double* %ptr, double %val) alwaysinline {
+entry:
+    %ptrval = load volatile double* %ptr
+    ; Check if val is a NaN and return *ptr early if so
+    %valnan = fcmp uno double %val, %val
+    br i1 %valnan, label %done, label %lt_check
+
+lt_check:
+    %dold = phi double [ %ptrval, %entry ], [ %dcas, %attempt ]
+    ; Continue attempts if dold < val or dold is NaN (using ult semantics)
+    %lt = fcmp ult double %dold, %val
+    br i1 %lt, label %attempt, label %done
+
+attempt:
+    ; Attempt to swap in the larger value
+    %iold = bitcast double %dold to i64
+    %iptr = bitcast double* %ptr to i64*
+    %ival = bitcast double %val to i64
+    %cas = cmpxchg volatile i64* %iptr, i64 %iold, i64 %ival monotonic
+    %dcas = bitcast i64 %cas to double
+    br label %lt_check
+
+done:
+    ; Return max
+    %ret = phi double [ %ptrval, %entry ], [ %dold, %lt_check ]
+    ret double %ret
+}
+"""
+
+
+def _replace_datalayout(llvmir):
+    """
+    Find the line containing the datalayout and replace it
+    """
+    lines = llvmir.splitlines()
+    for i, ln in enumerate(lines):
+        if ln.startswith("target datalayout"):
+            tmp = 'target datalayout = "{0}"'
+            lines[i] = tmp.format(default_data_layout)
+            break
+    return '\n'.join(lines)
+
 
 def llvm_to_ptx(llvmir, **opts):
     cu = CompilationUnit()
     libdevice = LibDevice(arch=opts.get('arch', 'compute_20'))
     # New LLVM generate a shorthand for datalayout that NVVM does not know
-    llvmir = llvmir.replace('e-i64:64-v16:16-v32:32-n16:32:64',
-                            default_data_layout)
-    # Replace with our cmpxchg implementation because LLVM 3.5 has a new
-    # semantic for cmpxchg.
-    llvmir = llvmir.replace('declare i32 @___numba_cas_hack(i32*, i32, i32)',
-                            ir_numba_cas_hack)
+    llvmir = _replace_datalayout(llvmir)
+    # Replace with our cmpxchg and atomic implementations because LLVM 3.5 has
+    # a new semantic for cmpxchg.
+    replacements = [
+        ('declare i32 @___numba_cas_hack(i32*, i32, i32)',
+            ir_numba_cas_hack),
+        ('declare double @___numba_atomic_double_add(double*, double)',
+            ir_numba_atomic_double_add),
+        ('declare double @___numba_atomic_double_max(double*, double)',
+            ir_numba_atomic_double_max)]
+
+    for decl, fn in replacements:
+        llvmir = llvmir.replace(decl, fn)
 
     llvmir = llvm33_to_32_ir(llvmir)
     cu.add_module(llvmir.encode('utf8'))
