@@ -6,7 +6,6 @@ import llvmlite.llvmpy.core as lc
 import llvmlite.llvmpy.ee as le
 import llvmlite.binding as ll
 from numba.targets.imputils import implement, Registry
-from numba.targets.npyimpl import register_casters
 from numba import cgutils
 from numba import types
 from .cudadrv import nvvm
@@ -15,7 +14,6 @@ from . import nvvmutils, stubs
 registry = Registry()
 register = registry.register
 
-register_casters(register)
 
 @register
 @implement('ptx.grid.1d', types.intp)
@@ -152,7 +150,8 @@ def ptx_cmem_arylike(context, builder, sig, args):
                            shape=cgutils.pack_array(builder, kshape),
                            strides=cgutils.pack_array(builder, kstrides),
                            itemsize=ary.itemsize,
-                           parent=ary.parent)
+                           parent=ary.parent,
+                           meminfo=None)
 
     return ary._getvalue()
 
@@ -405,6 +404,7 @@ def _make_array(context, builder, dataptr, dtype, shape, layout='C'):
     context.populate_array(ary,
                            data=builder.bitcast(dataptr, ary.data.type),
                            shape=cgutils.pack_array(builder, kshape),
-                           strides = cgutils.pack_array(builder, kstrides),
-                           itemsize=context.get_constant(types.intp, itemsize))
+                           strides=cgutils.pack_array(builder, kstrides),
+                           itemsize=context.get_constant(types.intp, itemsize),
+                           meminfo=None)
     return ary._getvalue()
