@@ -4,10 +4,10 @@ import functools
 import inspect
 import sys
 
-from numba import _dispatcher, compiler, utils
+from numba import _dispatcher, compiler, utils, types
 from numba.typeconv.rules import default_type_manager
 from numba import sigutils, serialize, types, typing
-from numba.typing.templates import resolve_overload
+from numba.typing.templates import resolve_overload, fold_arguments
 from numba.bytecode import get_code_object
 from numba.six import create_bound_method, next
 
@@ -19,9 +19,8 @@ class _OverloadedBase(_dispatcher.Dispatcher):
 
     __numba__ = "py_func"
 
-    def __init__(self, arg_count, py_func):
+    def __init__(self, arg_count, py_func, pysig):
         self._tm = default_type_manager
-        #_dispatcher.Dispatcher.__init__(self, self._tm.get_pointer(), arg_count)
 
         # A mapping of signatures to entry points
         self.overloads = {}
@@ -36,12 +35,19 @@ class _OverloadedBase(_dispatcher.Dispatcher):
         # but newer python uses a different name
         self.__code__ = self.func_code
 
-        self._pysig = utils.pysignature(self.py_func)
+        self._pysig = pysig
         argnames = tuple(self._pysig.parameters)
         defargs = self.py_func.__defaults__ or ()
+        try:
+            lastarg = list(self._pysig.parameters.values())[-1]
+        except IndexError:
+            has_stararg = False
+        else:
+            has_stararg = lastarg.kind == lastarg.VAR_POSITIONAL
         _dispatcher.Dispatcher.__init__(self, self._tm.get_pointer(),
-                                        arg_count, self.fold_args,
-                                        argnames, defargs)
+                                        arg_count, self._fold_args,
+                                        argnames, defargs,
+                                        has_stararg)
 
         self.doc = py_func.__doc__
         self._compile_lock = utils.NonReentrantLock()
@@ -114,19 +120,16 @@ class _OverloadedBase(_dispatcher.Dispatcher):
         *args* and *kws* types.  This allows to resolve the return type.
         """
         # Fold keyword arguments and resolve default values
-        ba = self._pysig.bind(*args, **kws)
-        for param in self._pysig.parameters.values():
-            name = param.name
-            default = param.default
-            if (default is not param.empty and
-                name not in ba.arguments):
-                ba.arguments[name] = self.typeof_pyval(default)
-        if ba.kwargs:
-            # There's a remaining keyword argument, e.g. if omitting
-            # some argument with a default value before it.
-            raise NotImplementedError("unhandled keyword argument: %s"
-                                      % list(ba.kwargs))
-        args = ba.args
+        def normal_handler(index, param, value):
+            return value
+        def default_handler(index, param, default):
+            return self.typeof_pyval(default)
+        def stararg_handler(index, param, values):
+            return types.Tuple(values)
+        args = fold_arguments(self._pysig, args, kws,
+                              normal_handler,
+                              default_handler,
+                              stararg_handler)
         kws = {}
         # Ensure an overload is available, but avoid compiler re-entrance
         if self._can_compile and not self.is_compiling:
@@ -208,11 +211,6 @@ class _OverloadedBase(_dispatcher.Dispatcher):
         This is called from numba._dispatcher as a fallback if the native code
         cannot decide the type.
         """
-        if isinstance(val, utils.INT_TYPES):
-            # Ensure no autoscaling of integer type, to match the
-            # typecode() function in _dispatcher.c.
-            return types.int64
-
         tp = self.typingctx.resolve_argument_type(val)
         if tp is None:
             tp = types.pyobject
@@ -226,7 +224,7 @@ class Overloaded(_OverloadedBase):
     This is an abstract base class. Subclasses should define the targetdescr
     class attribute.
     """
-    fold_args = True
+    _fold_args = True
 
     def __init__(self, py_func, locals={}, targetoptions={}):
         """
@@ -242,10 +240,10 @@ class Overloaded(_OverloadedBase):
         self.typingctx = self.targetdescr.typing_context
         self.targetctx = self.targetdescr.target_context
 
-        argspec = inspect.getargspec(py_func)
-        argct = len(argspec.args)
+        pysig = utils.pysignature(py_func)
+        arg_count = len(pysig.parameters)
 
-        _OverloadedBase.__init__(self, argct, py_func)
+        _OverloadedBase.__init__(self, arg_count, py_func, pysig)
 
         functools.update_wrapper(self, py_func)
 
@@ -333,16 +331,14 @@ class LiftedLoop(_OverloadedBase):
     Implementation of the hidden dispatcher objects used for lifted loop
     (a lifted loop is really compiled as a separate function).
     """
-    fold_args = False
+    _fold_args = False
 
     def __init__(self, bytecode, typingctx, targetctx, locals, flags):
         self.typingctx = typingctx
         self.targetctx = targetctx
 
-        argspec = bytecode.argspec
-        argct = len(argspec.args)
-
-        _OverloadedBase.__init__(self, argct, bytecode.func)
+        _OverloadedBase.__init__(self, bytecode.arg_count, bytecode.func,
+                                 bytecode.pysig)
 
         self.locals = locals
         self.flags = flags
