@@ -85,6 +85,12 @@ class BaseGeneratorLower(object):
         resume_index = self.context.get_constant(types.int32, 0)
         # Structure index #1: the function arguments
         argsty = retty.elements[1]
+
+        # Incref all NRT objects before storing into generator states
+        if self.context.enable_nrt:
+            for argty, argval in zip(self.fndesc.argtypes, lower.fnargs):
+                self.context.nrt_incref(builder, argty, argval)
+
         argsval = cgutils.make_anonymous_struct(builder, lower.fnargs,
                                                 argsty)
         gen_struct = cgutils.make_anonymous_struct(builder,
@@ -174,7 +180,6 @@ class BaseGeneratorLower(object):
         self.resume_blocks[index] = block
 
 
-
 class GeneratorLower(BaseGeneratorLower):
     """
     Support class for lowering nopython generators.
@@ -186,6 +191,46 @@ class GeneratorLower(BaseGeneratorLower):
     def box_generator_struct(self, lower, gen_struct):
         return gen_struct
 
+    def lower_finalize_func_body(self, builder, genptr):
+        """
+        Lower the body of the generator's finalizer: decref all live
+        state variables.
+        """
+        if self.context.enable_nrt:
+            resume_index_ptr = cgutils.gep(builder, genptr, 0, 0,
+                                           name='gen.resume_index')
+            resume_index = builder.load(resume_index_ptr)
+
+            # If resume_index is 0, next() was never called
+            # Note: The init func has to acquire the reference
+            #       of all arguments; otherwise, they will be destroyed at the
+            #       end of the init func.  The proper release of these
+            #       references relies on the actual NPM function, which is
+            #       never called if the generator exit before any entry into
+            #       the NPM function.
+            need_args_cleanup = builder.icmp_signed(
+                '==', resume_index, Constant.int(resume_index.type, 0))
+
+            with cgutils.ifthen(builder, need_args_cleanup):
+                gen_args_ptr = cgutils.gep(builder, genptr, 0, 1, name="gen_args")
+                assert len(self.fndesc.argtypes) == len(gen_args_ptr.type.pointee)
+                for elem_idx, argty in enumerate(self.fndesc.argtypes):
+                    argptr = cgutils.gep(builder, gen_args_ptr, 0, elem_idx)
+                    argval = builder.load(argptr)
+                    self.context.nrt_decref(builder, argty, argval)
+
+            # Always run the finalizer to clear the block
+            gen_state_ptr = cgutils.gep(builder, genptr, 0, 2, name='gen.state')
+
+            for state_index in range(len(self.gentype.state_types)):
+                state_slot = cgutils.gep(builder, gen_state_ptr,
+                                         0, state_index)
+                ty = self.gentype.state_types[state_index]
+                val = self.context.unpack_value(builder, ty, state_slot)
+                if self.context.enable_nrt:
+                    self.context.nrt_decref(builder, ty, val)
+
+        builder.ret_void()
 
 class PyGeneratorLower(BaseGeneratorLower):
     """
@@ -200,7 +245,7 @@ class PyGeneratorLower(BaseGeneratorLower):
         return types.Generator(
             gen_func=self.interp.bytecode.func,
             yield_type=types.pyobject,
-            arg_types=(types.pyobject,) * len(self.interp.argspec.args),
+            arg_types=(types.pyobject,) * self.interp.arg_count,
             state_types=(types.pyobject,) * len(self.geninfo.state_vars),
             has_finalizer=True,
             )
@@ -293,3 +338,6 @@ class LowerYield(object):
             ty = self.gentype.state_types[state_index]
             val = self.context.unpack_value(self.builder, ty, state_slot)
             self.lower.storevar(val, name)
+            # Previous storevar is making an extra incref
+            if self.context.enable_nrt:
+                self.lower.decref(ty, val)
