@@ -1,7 +1,9 @@
 from __future__ import print_function, division, absolute_import
 
+from collections import namedtuple
 import sys
 
+from llvmlite.ir import Value
 from llvmlite.llvmpy.core import Constant, Type, Builder
 
 
@@ -20,6 +22,9 @@ class ForbiddenConstruct(LoweringError):
     pass
 
 
+_VarArgItem = namedtuple("_VarArgItem", ("vararg", "index"))
+
+
 class BaseLower(object):
     """
     Lower IR to LLVM
@@ -187,6 +192,19 @@ class Lower(BaseLower):
             ty = self.typeof(inst.target.name)
             val = self.lower_assign(ty, inst)
             self.storevar(val, inst.target.name)
+            # TODO: emit incref/decref in the numba IR properly.
+            # Workaround due to lack of proper incref/decref info.
+            if self.context.enable_nrt:
+                if isinstance(inst.value, ir.Expr) and inst.value.op == 'call':
+                    callexpr = inst.value
+                    # NPM function returns new reference
+                    fnty = self.typeof(callexpr.func.name)
+                    if (isinstance(fnty, types.Dispatcher)
+                        or (isinstance(fnty, types.Function)
+                            and getattr(fnty.template,
+                                        'return_new_reference',
+                                        False))):
+                        self.decref(ty, val)
 
         elif isinstance(inst, ir.Branch):
             cond = self.loadvar(inst.cond.name)
@@ -247,7 +265,14 @@ class Lower(BaseLower):
             return impl(self.builder, (target, index, value))
 
         elif isinstance(inst, ir.Del):
-            pass
+            try:
+                # XXX: incorrect Del injection?
+                val = self.loadvar(inst.value)
+            except KeyError:
+                pass
+            else:
+                self.decref(self.typeof(inst.value), val)
+                self._delete_variable(inst.value)
 
         elif isinstance(inst, ir.SetAttr):
             target = self.loadvar(inst.target.name)
@@ -302,10 +327,7 @@ class Lower(BaseLower):
                 return self.context.get_constant_generic(self.builder, ty,
                                                          value.value)
 
-            elif (isinstance(ty, types.Dummy) or
-                    isinstance(ty, types.Module) or
-                    isinstance(ty, types.Function) or
-                    isinstance(ty, types.Dispatcher)):
+            elif isinstance(ty, types.Dummy):
                 return self.context.get_dummy_value()
 
             elif isinstance(ty, types.Array):
@@ -366,6 +388,20 @@ class Lower(BaseLower):
         return self.context.cast(self.builder, res, signature.return_type,
                                  resty)
 
+    def _cast_var(self, var, ty):
+        """
+        Cast a Numba IR variable to the given Numba type, returning a
+        low-level value.
+        """
+        if isinstance(var, _VarArgItem):
+            varty = self.typeof(var.vararg.name)[var.index]
+            val = self.builder.extract_value(self.loadvar(var.vararg.name),
+                                             var.index)
+        else:
+            varty = self.typeof(var.name)
+            val = self.loadvar(var.name)
+        return self.context.cast(self.builder, val, varty, ty)
+
     def lower_call(self, resty, expr):
         signature = self.fndesc.calltypes[expr]
         if isinstance(signature.return_type, types.Phantom):
@@ -376,40 +412,38 @@ class Lower(BaseLower):
             argvals = expr.func.args
         else:
             fnty = self.typeof(expr.func.name)
+            pos_args = expr.args
+            if expr.vararg:
+                # Inject *args from function call
+                # The lowering will be done in _cast_var() above.
+                tp_vararg = self.typeof(expr.vararg.name)
+                assert isinstance(tp_vararg, types.BaseTuple)
+                pos_args = pos_args + [_VarArgItem(expr.vararg, i)
+                                       for i in range(len(tp_vararg))]
 
             # Fold keyword arguments and resolve default argument values
-            defargs = set()
-            try:
-                pysig = fnty.pysig
-            except AttributeError:
+            pysig = signature.pysig
+            if pysig is None:
                 if expr.kws:
                     raise NotImplementedError("unsupported keyword arguments "
                                               "when calling %s" % (fnty,))
-                args = expr.args
+                argvals = [self._cast_var(var, sigty)
+                           for var, sigty in zip(pos_args, signature.args)]
             else:
-                ba = pysig.bind(*expr.args, **dict(expr.kws))
-                for i, param in enumerate(pysig.parameters.values()):
-                    name = param.name
-                    default = param.default
-                    if (default is not param.empty and
-                        name not in ba.arguments):
-                        value = self.context.get_constant_generic(
-                            self.builder, signature.args[i], default)
-                        ba.arguments[name] = value
-                        defargs.add(i)
-                # This is ensured in Dispatcher.get_call_template()
-                assert not ba.kwargs
-                args = ba.args
-
-            # Fetch and cast non-default arguments
-            argvals = list(args)
-            for i, a in enumerate(argvals):
-                if i not in defargs:
-                    ty = self.typeof(a.name)
-                    val = self.loadvar(a.name)
-                    a = self.context.cast(self.builder, val, ty,
-                                          signature.args[i])
-                    argvals[i] = a
+                def normal_handler(index, param, var):
+                    return self._cast_var(var, signature.args[index])
+                def default_handler(index, param, default):
+                    return self.context.get_constant_generic(
+                                self.builder, signature.args[index], default)
+                def stararg_handler(index, param, vars):
+                    values = [self._cast_var(var, sigty)
+                              for var, sigty in zip(vars, signature.args[index])]
+                    return cgutils.make_anonymous_struct(self.builder, values)
+                argvals = typing.fold_arguments(pysig,
+                                                pos_args, dict(expr.kws),
+                                                normal_handler,
+                                                default_handler,
+                                                stararg_handler)
 
         if isinstance(fnty, types.ExternalFunction):
             # Handle a named external function
@@ -429,8 +463,37 @@ class Lower(BaseLower):
         elif isinstance(fnty, types.ExternalFunctionPointer):
             # Handle a C function pointer
             pointer = self.loadvar(expr.func.name)
-            res = self.context.call_function_pointer(self.builder, pointer,
-                                                     argvals, fnty.cconv)
+            # If the external function pointer uses libpython
+            if fnty.requires_gil:
+                pyapi = self.context.get_python_api(self.builder)
+                # Acquire the GIL
+                gil_state = pyapi.gil_ensure()
+                # Make PyObjects
+                newargvals = []
+                pyvals = []
+                for exptyp, gottyp, aval in zip(fnty.sig.args, signature.args,
+                                                argvals):
+                    # Adjust argument values to pyobjects
+                    if exptyp == types.ffi_forced_object:
+                        obj = pyapi.from_native_value(aval, gottyp)
+                        newargvals.append(obj)
+                        pyvals.append(obj)
+                    else:
+                        newargvals.append(aval)
+
+                # Call external function
+                res = self.context.call_function_pointer(self.builder, pointer,
+                                                         newargvals, fnty.cconv)
+                # Release PyObjects
+                for obj in pyvals:
+                    pyapi.decref(obj)
+
+                # Release the GIL
+                pyapi.gil_release(gil_state)
+            # If the external function pointer does NOT use libpython
+            else:
+                res = self.context.call_function_pointer(self.builder, pointer,
+                                                         argvals, fnty.cconv)
 
         else:
             # Normal function resolution (for Numba-compiled functions)
@@ -497,9 +560,9 @@ class Lower(BaseLower):
         elif expr.op == 'exhaust_iter':
             val = self.loadvar(expr.value.name)
             ty = self.typeof(expr.value.name)
-            # If we have a heterogenous tuple, we needn't do anything,
-            # and we can't iterate over it anyway.
-            if isinstance(ty, types.Tuple):
+            # If we have a tuple, we needn't do anything
+            # (and we can't iterate over the heterogenous ones).
+            if isinstance(ty, types.BaseTuple):
                 return val
 
             itemty = ty.iterator_type.yield_type
@@ -602,6 +665,9 @@ class Lower(BaseLower):
             castval = self.context.cast(self.builder, val, ty, resty)
             return castval
 
+        elif expr.op in self.context.special_ops:
+            return self.context.special_ops[expr.op](self, expr)
+
         raise NotImplementedError(expr)
 
     def getvar(self, name):
@@ -612,16 +678,58 @@ class Lower(BaseLower):
         return self.builder.load(ptr)
 
     def storevar(self, value, name):
+        fetype = self.typeof(name)
+        # Clean up existing value stored in the variable
+        try:
+            # Load original value in variable
+            old = self.loadvar(name)
+        except KeyError:
+            # If it has not been defined, don't do anything
+            pass
+        else:
+            # Else, dereference the old value
+            self.decref(fetype, old)
+        # Store variable
         if name not in self.varmap:
-            self.varmap[name] = self.alloca_lltype(name, value.type)
+            # If not already defined, allocate it
+            llty = self.context.get_value_type(fetype)
+            ptr = self.alloca_lltype(name, llty)
+            # Remember the pointer
+            self.varmap[name] = ptr
+
         ptr = self.getvar(name)
-        assert value.type == ptr.type.pointee,\
-            "store %s to ptr of %s" % (value.type, ptr.type.pointee)
+        if value.type != ptr.type.pointee:
+            msg = ("Storing {value.type} to ptr of {ptr.type.pointee}. "
+                   "FE type {fetype}").format(value=value, ptr=ptr,
+                                              fetype=fetype)
+            raise AssertionError(msg)
+
         self.builder.store(value, ptr)
+        # Incref
+        self.incref(fetype, value)
 
     def alloca(self, name, type):
         lltype = self.context.get_value_type(type)
         return self.alloca_lltype(name, lltype)
 
     def alloca_lltype(self, name, lltype):
-        return cgutils.alloca_once(self.builder, lltype, name=name)
+        return cgutils.alloca_once(self.builder, lltype, name=name, zfill=True)
+
+    def incref(self, typ, val):
+        if not self.context.enable_nrt:
+            return
+
+        self.context.nrt_incref(self.builder, typ, val)
+
+    def decref(self, typ, val):
+        if not self.context.enable_nrt:
+            return
+
+        self.context.nrt_decref(self.builder, typ, val)
+
+    def _delete_variable(self, varname):
+        """
+        Zero-fill variable to avoid crashing due to extra ir.Del
+        """
+        storage = self.getvar(varname)
+        self.builder.store(Constant.null(storage.type.pointee), storage)
