@@ -214,7 +214,6 @@ def build_ufunc_wrapper(library, context, func, signature, objmode, env):
     # Run optimizer
     library.add_ir_module(wrapper_module)
     wrapper = library.get_function(wrapper.name)
-    oldfunc.linkage = LINKAGE_INTERNAL
 
     return wrapper
 
@@ -564,7 +563,8 @@ class GUArrayArg(object):
                                shape=shape,
                                strides=strides,
                                itemsize=context.get_constant(types.intp,
-                                                             itemsize))
+                                                             itemsize),
+                               meminfo=None)
 
         return array._getvalue()
 
