@@ -10,7 +10,8 @@ import llvmlite.llvmpy.core as lc
 
 from numba.config import PYVERSION
 import numba.ctypes_support as ctypes
-from numba import types, utils, cgutils, _helperlib, assume
+from numba import numpy_support
+from numba import types, utils, cgutils, _helperlib
 
 
 class NativeValue(object):
@@ -690,6 +691,42 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name=fname)
         return self.builder.call(fn, [strobj])
 
+    def string_as_string_and_size(self, strobj):
+        """
+        Returns a tuple of ``(ok, buffer, length)``.
+        The ``ok`` is i1 value that is set if ok.
+        The ``buffer`` is a i8* of the output buffer.
+        The ``length`` is a i32/i64 (py_ssize_t) of the length of the buffer.
+        """
+
+        p_length = cgutils.alloca_once(self.builder, self.py_ssize_t)
+        if PYVERSION >= (3, 0):
+            fnty = Type.function(self.cstring, [self.pyobj,
+                                                self.py_ssize_t.as_pointer()])
+            fname = "PyUnicode_AsUTF8AndSize"
+            fn = self._get_function(fnty, name=fname)
+
+            buffer = self.builder.call(fn, [strobj, p_length])
+            ok = self.builder.icmp_unsigned('!=',
+                                            ir.Constant(buffer.type, None),
+                                            buffer)
+        else:
+            fnty = Type.function(lc.Type.int(), [self.pyobj,
+                                                 self.cstring.as_pointer(),
+                                                 self.py_ssize_t.as_pointer()])
+            fname = "PyString_AsStringAndSize"
+            fn = self._get_function(fnty, name=fname)
+            # Allocate space for the output parameters
+            p_buffer = cgutils.alloca_once(self.builder, self.cstring)
+
+            status = self.builder.call(fn, [strobj, p_buffer, p_length])
+
+            negone = ir.Constant(status.type, -1)
+            ok = self.builder.icmp_signed("!=", status, negone)
+            buffer = self.builder.load(p_buffer)
+
+        return (ok, buffer, self.builder.load(p_length))
+
     def string_from_string_and_size(self, string, size):
         fnty = Type.function(self.pyobj, [self.cstring, self.py_ssize_t])
         if PYVERSION >= (3, 0):
@@ -909,7 +946,13 @@ class PythonAPI(object):
 
         elif isinstance(typ, types.Array):
             val, failed = self.to_native_array(obj, typ)
-            return NativeValue(val, is_error=failed)
+            val_on_stack = cgutils.alloca_once_value(builder, val)
+
+            def cleanup_array():
+                val = self.builder.load(val_on_stack)
+
+            return NativeValue(val, is_error=failed,
+                               cleanup=cleanup_array)
 
         elif isinstance(typ, types.Buffer):
             return self.to_native_buffer(obj, typ)
@@ -943,10 +986,41 @@ class PythonAPI(object):
                         builder.store(builder.bitcast(ptr, ptrty), ret)
                 return NativeValue(builder.load(ret), is_error=c_api_error())
 
+        elif isinstance(typ, types.CharSeq):
+            lty = self.context.get_value_type(typ)
+            ok, buffer, size = self.string_as_string_and_size(obj)
+
+            # Check if the returned string size fits in the charseq
+            storage_size = ir.Constant(size.type, typ.count)
+            size_fits = builder.icmp_unsigned("<=", size, storage_size)
+
+            # Allow truncation of string
+            size = builder.select(size_fits, size, storage_size)
+
+            # Initialize output to zero bytes
+            null_string = ir.Constant(lty, None)
+            outspace  = cgutils.alloca_once_value(builder, null_string)
+
+            # If conversion is ok, copy the buffer to the output storage.
+            with cgutils.if_likely(builder, ok):
+                # We don't need to set the NULL-terminator because the storage
+                # is already zero-filled.
+                cgutils.memcpy(builder, builder.bitcast(outspace, buffer.type),
+                               buffer, size)
+
+            ret = builder.load(outspace)
+            return NativeValue(ret, is_error=builder.not_(ok))
+
         raise NotImplementedError("cannot convert %s to native value" % (typ,))
 
     def from_native_return(self, val, typ):
-        return self.from_native_value(val, typ)
+        assert not isinstance(typ, types.Optional), "callconv should have " \
+                                                    "prevented the return of " \
+                                                    "optional value"
+        out = self.from_native_value(val, typ)
+        if self.context.enable_nrt:
+            self.context.nrt_decref(self.builder, typ, val)
+        return out
 
     def from_native_value(self, val, typ):
         if typ == types.pyobject:
@@ -1018,6 +1092,14 @@ class PythonAPI(object):
         elif isinstance(typ, types.Generator):
             return self.from_native_generator(val, typ)
 
+        elif isinstance(typ, types.CharSeq):
+            return self.from_native_charseq(val, typ)
+
+        elif typ == types.voidptr:
+            ll_intp = self.context.get_value_type(types.uintp)
+            addr = self.builder.ptrtoint(val, ll_intp)
+            return self.from_native_value(addr, types.uintp)
+
         raise NotImplementedError(typ)
 
     def to_native_int(self, obj, typ):
@@ -1044,7 +1126,10 @@ class PythonAPI(object):
 
         with cgutils.if_likely(self.builder, self.builder.not_(is_error)):
             ptr = self.builder.bitcast(aryptr, self.voidptr)
-            self.numba_buffer_adaptor(buf, ptr)
+            if self.context.enable_nrt:
+                self.nrt_adapt_buffer_from_python(buf, ptr)
+            else:
+                self.numba_buffer_adaptor(buf, ptr)
 
         def cleanup():
             self.release_buffer(buf)
@@ -1060,17 +1145,24 @@ class PythonAPI(object):
         nativeary = nativearycls(self.context, self.builder)
         aryptr = nativeary._getpointer()
         ptr = self.builder.bitcast(aryptr, self.voidptr)
-        errcode = self.numba_array_adaptor(ary, ptr)
+        if self.context.enable_nrt:
+            errcode = self.nrt_adapt_ndarray_from_python(ary, ptr)
+        else:
+            errcode = self.numba_array_adaptor(ary, ptr)
         failed = cgutils.is_not_null(self.builder, errcode)
         return self.builder.load(aryptr), failed
 
     def from_native_array(self, ary, typ):
-        assert assume.return_argument_array_only
+        builder = self.builder
         nativearycls = self.context.make_array(typ)
-        nativeary = nativearycls(self.context, self.builder, value=ary)
-        parent = nativeary.parent
-        self.incref(parent)
-        return parent
+        nativeary = nativearycls(self.context, builder, value=ary)
+        if self.context.enable_nrt:
+            newary = self.nrt_adapt_ndarray_to_python(typ, ary)
+            return newary
+        else:
+            parent = nativeary.parent
+            self.incref(parent)
+            return parent
 
     def to_native_optional(self, obj, typ):
         """
@@ -1198,7 +1290,76 @@ class PythonAPI(object):
         return self.builder.call(fn,
                                  (state_size, initial_state, genfn, finalizer, env))
 
+    def nrt_adapt_ndarray_to_python(self, aryty, ary):
+        assert self.context.enable_nrt, "NRT required"
+        np_dtype = numpy_support.as_dtype(aryty.dtype)
+
+        intty = ir.IntType(32)
+        fnty = Type.function(self.pyobj, [self.voidptr, intty, self.pyobj])
+        fn = self._get_function(fnty, name="NRT_adapt_ndarray_to_python")
+        fn.args[0].add_attribute(lc.ATTR_NO_CAPTURE)
+        dtype = numpy_support.as_dtype(aryty.dtype)
+
+        ndim = self.context.get_constant(types.int32, aryty.ndim)
+        dtypeptr = self.unserialize(self.serialize_object(np_dtype))
+
+        aryptr = cgutils.alloca_once_value(self.builder, ary)
+        return self.builder.call(fn, [self.builder.bitcast(aryptr,
+                                                           self.voidptr),
+                                      ndim, dtypeptr])
+
+    def nrt_adapt_ndarray_from_python(self, ary, ptr):
+        assert self.context.enable_nrt
+        fnty = Type.function(Type.int(), [self.pyobj, self.voidptr])
+        fn = self._get_function(fnty, name="NRT_adapt_ndarray_from_python")
+        fn.args[0].add_attribute(lc.ATTR_NO_CAPTURE)
+        fn.args[1].add_attribute(lc.ATTR_NO_CAPTURE)
+        return self.builder.call(fn, (ary, ptr))
+
+    def nrt_adapt_buffer_from_python(self, buf, ptr):
+        assert self.context.enable_nrt
+        fnty = Type.function(Type.void(), [Type.pointer(self.py_buffer_t),
+                                           self.voidptr])
+        fn = self._get_function(fnty, name="NRT_adapt_buffer_from_python")
+        fn.args[0].add_attribute(lc.ATTR_NO_CAPTURE)
+        fn.args[1].add_attribute(lc.ATTR_NO_CAPTURE)
+        return self.builder.call(fn, (buf, ptr))
+
+    def from_native_charseq(self, val, typ):
+        builder = self.builder
+        rawptr = cgutils.alloca_once_value(builder, value=val)
+        strptr = builder.bitcast(rawptr, self.cstring)
+        fullsize = self.context.get_constant(types.intp, typ.count)
+        zero = self.context.get_constant(types.intp, 0)
+        count = cgutils.alloca_once_value(builder, zero)
+
+        bbend = cgutils.append_basic_block(builder, "end.string.count")
+
+        # Find the length of the string
+        with cgutils.loop_nest(builder, [fullsize], fullsize.type) as [idx]:
+            # Get char at idx
+            ch = builder.load(builder.gep(strptr, [idx]))
+            # Store the current index as count
+            builder.store(idx, count)
+            # Check if the char is a null-byte
+            ch_is_null = cgutils.is_null(builder, ch)
+            # If the char is a null-byte
+            with cgutils.ifthen(builder, ch_is_null):
+                # Jump to the end
+                builder.branch(bbend)
+
+        # This is reached if there is no null-byte in the string
+        # Then, set count to the fullsize
+        builder.store(fullsize, count)
+        # Jump to the end
+        builder.branch(bbend)
+
+        builder.position_at_end(bbend)
+        strlen = builder.load(count)
+        return self.bytes_from_string_and_size(strptr, strlen)
+
     def numba_array_adaptor(self, ary, ptr):
+        assert not self.context.enable_nrt
         fnty = Type.function(Type.int(), [self.pyobj, self.voidptr])
         fn = self._get_function(fnty, name="numba_adapt_ndarray")
         fn.args[0].add_attribute(lc.ATTR_NO_CAPTURE)
