@@ -6,6 +6,7 @@ the buffer protocol.
 from __future__ import print_function, absolute_import, division
 
 from functools import reduce
+import math
 
 import llvmlite.llvmpy.core as lc
 
@@ -13,6 +14,8 @@ import numba.ctypes_support as ctypes
 import numpy
 from llvmlite.llvmpy.core import Constant
 from numba import types, cgutils
+from numba.numpy_support import as_dtype
+from numba.numpy_support import version as numpy_version
 from numba.targets.imputils import (builtin, builtin_attr, implement,
                                     impl_attribute, impl_attribute_generic,
                                     iterator_impl, iternext_impl,
@@ -28,7 +31,8 @@ def make_array(array_type):
     return cgutils.create_struct_proxy(array_type)
 
 
-def populate_array(array, data, shape, strides, itemsize, parent=None):
+def populate_array(array, data, shape, strides, itemsize, meminfo,
+                   parent=None):
     """
     Helper function for populating array structures.
     This avoids forgetting to set fields.
@@ -38,10 +42,15 @@ def populate_array(array, data, shape, strides, itemsize, parent=None):
     datamodel = array._datamodel
     required_fields = set(datamodel._fields)
 
+    if meminfo is None:
+        meminfo = Constant.null(context.get_value_type(
+            datamodel.get_type('meminfo')))
+
     attrs = dict(shape=shape,
                  strides=strides,
                  data=data,
-                 itemsize=itemsize)
+                 itemsize=itemsize,
+                 meminfo=meminfo,)
 
     # Set `parent` attribute
     if parent is None:
@@ -73,32 +82,6 @@ def populate_array(array, data, shape, strides, itemsize, parent=None):
     return array
 
 
-def make_array_ctype(ndim):
-    """Create a ctypes representation of an array_type.
-
-    Parameters
-    -----------
-    ndim: int
-        number of dimensions of array
-
-    Returns
-    -----------
-        a ctypes array structure for an array with the given number of
-        dimensions
-    """
-    c_intp = ctypes.c_ssize_t
-
-    class c_array(ctypes.Structure):
-        _fields_ = [('parent', ctypes.c_void_p),
-                    ('nitems', c_intp),
-                    ('itemsize', c_intp),
-                    ('data', ctypes.c_void_p),
-                    ('shape', c_intp * ndim),
-                    ('strides', c_intp * ndim)]
-
-    return c_array
-
-
 @struct_factory(types.ArrayIterator)
 def make_arrayiter_cls(iterator_type):
     """
@@ -182,7 +165,9 @@ def getitem_arraynd_intp(context, builder, sig, args):
                        shape=cgutils.pack_array(builder, in_shapes[1:]),
                        strides=cgutils.pack_array(builder, in_strides[1:]),
                        itemsize=adapted_ary.itemsize,
+                       meminfo=adapted_ary.meminfo,
                        parent=adapted_ary.parent,)
+
         result = out_ary._getvalue()
     else:
         raise NotImplementedError("1D indexing into %dD array" % aryty.ndim)
@@ -222,6 +207,7 @@ def getitem_array1d_slice(context, builder, sig, args):
                    shape=cgutils.pack_array(builder, [shape]),
                    strides=cgutils.pack_array(builder, [stride]),
                    itemsize=ary.itemsize,
+                   meminfo=ary.meminfo,
                    parent=ary.parent)
 
     return retary._getvalue()
@@ -261,6 +247,7 @@ def getitem_array_unituple(context, builder, sig, args):
                        shape=cgutils.pack_array(builder, shapes),
                        strides=cgutils.pack_array(builder, strides),
                        itemsize=ary.itemsize,
+                       meminfo=ary.meminfo,
                        parent=ary.parent)
         return retary._getvalue()
     else:
@@ -317,6 +304,7 @@ def getitem_array_tuple(context, builder, sig, args):
                        shape=cgutils.pack_array(builder, shapes),
                        strides=cgutils.pack_array(builder, strides),
                        itemsize=ary.itemsize,
+                       meminfo=ary.meminfo,
                        parent=ary.parent)
         return retary._getvalue()
     else:
@@ -506,6 +494,50 @@ def array_prod(context, builder, sig, args):
                                     locals=dict(c=sig.return_type))
 
 
+@builtin
+@implement(numpy.cumsum, types.Kind(types.Array))
+@implement("array.cumsum", types.Kind(types.Array))
+def array_cumsum(context, builder, sig, args):
+    scalar_dtype = sig.return_type.dtype
+    dtype = getattr(numpy, str(scalar_dtype))
+
+    def array_cumsum_impl(arr):
+        size = 1
+        for i in arr.shape:
+            size = size * i
+        out = numpy.empty(size, dtype)
+        c = 0
+        for idx, v in enumerate(arr.flat):
+            c += v
+            out[idx] = c
+        return out
+
+    return context.compile_internal(builder, array_cumsum_impl, sig, args,
+                                    locals=dict(c=scalar_dtype))
+
+
+@builtin
+@implement(numpy.cumprod, types.Kind(types.Array))
+@implement("array.cumprod", types.Kind(types.Array))
+def array_cumsum(context, builder, sig, args):
+    scalar_dtype = sig.return_type.dtype
+    dtype = getattr(numpy, str(scalar_dtype))
+
+    def array_cumprod_impl(arr):
+        size = 1
+        for i in arr.shape:
+            size = size * i
+        out = numpy.empty(size, dtype)
+        c = 1
+        for idx, v in enumerate(arr.flat):
+            c *= v
+            out[idx] = c
+        return out
+
+    return context.compile_internal(builder, array_cumprod_impl, sig, args,
+                                    locals=dict(c=scalar_dtype))
+
+
 @builtin
 @implement(numpy.mean, types.Kind(types.Array))
 @implement("array.mean", types.Kind(types.Array))
@@ -622,6 +654,92 @@ def array_argmax(context, builder, sig, args):
     return context.compile_internal(builder, array_argmax_impl, sig, args)
 
 
+def _np_round_intrinsic(tp):
+    # np.round() always rounds half to even
+    return "llvm.rint.f%d" % (tp.bitwidth,)
+
+def _np_round_float(context, builder, tp, val):
+    llty = context.get_value_type(tp)
+    module = cgutils.get_module(builder)
+    fnty = lc.Type.function(llty, [llty])
+    fn = module.get_or_insert_function(fnty, name=_np_round_intrinsic(tp))
+    return builder.call(fn, (val,))
+
+@builtin
+@implement(numpy.round, types.Kind(types.Float))
+def scalar_round_unary(context, builder, sig, args):
+    return _np_round_float(context, builder, sig.args[0], args[0])
+
+@builtin
+@implement(numpy.round, types.Kind(types.Integer))
+def scalar_round_unary(context, builder, sig, args):
+    return args[0]
+
+@builtin
+@implement(numpy.round, types.Kind(types.Complex))
+def scalar_round_unary_complex(context, builder, sig, args):
+    fltty = sig.args[0].underlying_float
+    cplx_cls = context.make_complex(sig.args[0])
+    z = cplx_cls(context, builder, args[0])
+    z.real = _np_round_float(context, builder, fltty, z.real)
+    z.imag = _np_round_float(context, builder, fltty, z.imag)
+    return z._getvalue()
+
+@builtin
+@implement(numpy.round, types.Kind(types.Float), types.Kind(types.Integer))
+@implement(numpy.round, types.Kind(types.Integer), types.Kind(types.Integer))
+def scalar_round_binary_float(context, builder, sig, args):
+    def round_ndigits(x, ndigits):
+        if math.isinf(x) or math.isnan(x):
+            return x
+
+        # NOTE: this is CPython's algorithm, but perhaps this is overkill
+        # when emulating Numpy's behaviour.
+        if ndigits >= 0:
+            if ndigits > 22:
+                # pow1 and pow2 are each safe from overflow, but
+                # pow1*pow2 ~= pow(10.0, ndigits) might overflow.
+                pow1 = 10.0 ** (ndigits - 22)
+                pow2 = 1e22
+            else:
+                pow1 = 10.0 ** ndigits
+                pow2 = 1.0
+            y = (x * pow1) * pow2
+            if math.isinf(y):
+                return x
+            return (numpy.round(y) / pow2) / pow1
+
+        else:
+            pow1 = 10.0 ** (-ndigits)
+            y = x / pow1
+            return numpy.round(y) * pow1
+
+    return context.compile_internal(builder, round_ndigits, sig, args)
+
+@builtin
+@implement(numpy.round, types.Kind(types.Complex), types.Kind(types.Integer))
+def scalar_round_binary_complex(context, builder, sig, args):
+    def round_ndigits(z, ndigits):
+        return complex(numpy.round(z.real, ndigits),
+                       numpy.round(z.imag, ndigits))
+
+    return context.compile_internal(builder, round_ndigits, sig, args)
+
+
+@builtin
+@implement(numpy.round, types.Kind(types.Array), types.Kind(types.Integer),
+           types.Kind(types.Array))
+def array_round(context, builder, sig, args):
+    def array_round_impl(arr, decimals, out):
+        if arr.shape != out.shape:
+            raise ValueError("invalid output shape")
+        for index, val in numpy.ndenumerate(arr):
+            out[index] = numpy.round(val, decimals)
+        return out
+
+    return context.compile_internal(builder, array_round_impl, sig, args)
+
+
 #-------------------------------------------------------------------------------
 
 
@@ -701,6 +819,29 @@ def array_readonly(context, builder, typ, value):
     return context.get_constant(types.boolean, not typ.mutable)
 
 
+@builtin_attr
+@impl_attribute(types.Kind(types.Array), "ctypes",
+                types.Kind(types.ArrayCTypes))
+def array_ctypes(context, builder, typ, value):
+    arrayty = make_array(typ)
+    array = arrayty(context, builder, value)
+    # Cast void* data to uintp
+    addr = builder.ptrtoint(array.data, context.get_value_type(types.uintp))
+    # Create new ArrayCType structure
+    ctinfo_type = cgutils.create_struct_proxy(types.ArrayCTypes(typ))
+    ctinfo = ctinfo_type(context, builder)
+    ctinfo.data = addr
+    return ctinfo._getvalue()
+
+
+@builtin_attr
+@impl_attribute(types.Kind(types.ArrayCTypes), "data", types.uintp)
+def array_ctypes_data(context, builder, typ, value):
+    ctinfo_type = cgutils.create_struct_proxy(typ)
+    ctinfo = ctinfo_type(context, builder, value=value)
+    return ctinfo.data
+
+
 @builtin_attr
 @impl_attribute_generic(types.Kind(types.Array))
 def array_record_getattr(context, builder, typ, value, attr):
@@ -730,11 +871,11 @@ def array_record_getattr(context, builder, typ, value, attr):
                    shape=array.shape,
                    strides=array.strides,
                    itemsize=context.get_constant(types.intp, datasize),
+                   meminfo=array.meminfo,
                    parent=array.parent)
     return rary._getvalue()
 
 
-
 #-------------------------------------------------------------------------------
 # builtin `numpy.flat` implementation
 
@@ -795,14 +936,10 @@ def make_ndindex_cls(nditerty):
     """
     ndim = nditerty.ndim
 
-    class NdIndexIter(cgutils.Structure):
+    class NdIndexIter(cgutils.create_struct_proxy(nditerty)):
         """
         .ndindex() implementation.
         """
-        _fields = [('shape', types.UniTuple(types.intp, ndim)),
-                   ('indices', types.CPointer(types.intp)),
-                   ('exhausted', types.CPointer(types.boolean)),
-                   ]
 
         def init_specific(self, context, builder, shapes):
             zero = context.get_constant(types.intp, 0)
@@ -859,16 +996,10 @@ def _make_flattening_iter_cls(flatiterty, kind):
     dtype = array_type.dtype
 
     if array_type.layout == 'C':
-        class CContiguousFlatIter(cgutils.Structure):
+        class CContiguousFlatIter(cgutils.create_struct_proxy(flatiterty)):
             """
             .flat() / .ndenumerate() implementation for C-contiguous arrays.
             """
-            _fields = [('array', types.CPointer(array_type)),
-                       ('stride', types.intp),
-                       ('pointer', types.CPointer(types.CPointer(dtype))),
-                       ('index', types.CPointer(types.intp)),
-                       ('indices', types.CPointer(types.intp)),
-                       ]
 
             def init_specific(self, context, builder, arrty, arr):
                 zero = context.get_constant(types.intp, 0)
@@ -925,18 +1056,13 @@ def _make_flattening_iter_cls(flatiterty, kind):
         return CContiguousFlatIter
 
     else:
-        class FlatIter(cgutils.Structure):
+        class FlatIter(cgutils.create_struct_proxy(flatiterty)):
             """
             Generic .flat() / .ndenumerate() implementation for
             non-contiguous arrays.
             It keeps track of pointers along each dimension in order to
             minimize computations.
             """
-            _fields = [('array', types.CPointer(array_type)),
-                       ('pointers', types.CPointer(types.CPointer(dtype))),
-                       ('indices', types.CPointer(types.intp)),
-                       ('exhausted', types.CPointer(types.boolean)),
-                       ]
 
             def init_specific(self, context, builder, arrty, arr):
                 zero = context.get_constant(types.intp, 0)
@@ -1151,3 +1277,401 @@ def iternext_numpy_ndindex(context, builder, sig, args, result):
     nditer = nditercls(context, builder, value=nditer)
 
     nditer.iternext_specific(context, builder, result)
+
+
+# -----------------------------------------------------------------------------
+# Numpy array constructors
+
+def _empty_nd_impl(context, builder, arrtype, shapes):
+    """Utility function used for allocating a new array during LLVM code
+    generation (lowering).  Given a target context, builder, array
+    type, and a tuple or list of lowered dimension sizes, returns a
+    LLVM value pointing at a Numba runtime allocated array.
+    """
+    arycls = make_array(arrtype)
+    ary = arycls(context, builder)
+
+    datatype = context.get_data_type(arrtype.dtype)
+    itemsize = context.get_constant(types.intp,
+                                    context.get_abi_sizeof(datatype))
+
+    # compute array length
+    arrlen = context.get_constant(types.intp, 1)
+    for s in shapes:
+        arrlen = builder.mul(arrlen, s)
+
+    if arrtype.layout == 'C':
+        strides = [itemsize]
+        for dimension_size in reversed(shapes[1:]):
+            strides.append(builder.mul(strides[-1], dimension_size))
+        strides = tuple(reversed(strides))
+    elif arrtype.layout == 'F':
+        strides = [itemsize]
+        for dimension_size in shapes[:-1]:
+            strides.append(builder.mul(strides[-1], dimension_size))
+        strides = tuple(strides)
+    else:
+        raise NotImplementedError(
+            "Don't know how to allocate array with layout '{0}'.".format(
+                arrtype.layout))
+
+    meminfo = context.nrt_meminfo_alloc(builder,
+                                        size=builder.mul(itemsize, arrlen))
+    data = context.nrt_meminfo_data(builder, meminfo)
+
+    populate_array(ary,
+                   data=builder.bitcast(data, datatype.as_pointer()),
+                   shape=cgutils.pack_array(builder, shapes),
+                   strides=cgutils.pack_array(builder, strides),
+                   itemsize=itemsize,
+                   meminfo=meminfo)
+
+    return ary
+
+def _zero_fill_array(context, builder, ary):
+    """
+    Zero-fill an array.  The array must be contiguous.
+    """
+    cgutils.memset(builder, ary.data, builder.mul(ary.itemsize, ary.nitems), 0)
+
+
+def _parse_empty_args(context, builder, sig, args):
+    """
+    Parse the arguments of a np.empty(), np.zeros() or np.ones() call.
+    """
+    arrshapetype = sig.args[0]
+    arrshape = args[0]
+    arrtype = sig.return_type
+
+    if isinstance(arrshapetype, types.Integer):
+        ndim = 1
+        shapes = [context.cast(builder, arrshape, arrshapetype, types.intp)]
+    else:
+        ndim = arrshapetype.count
+        arrshape = context.cast(builder, arrshape, arrshapetype,
+                                types.UniTuple(types.intp, ndim))
+        shapes = cgutils.unpack_tuple(builder, arrshape, count=ndim)
+
+    zero = context.get_constant_generic(builder, types.intp, 0)
+    for dim in range(ndim):
+        is_neg = builder.icmp_signed('<', shapes[dim], zero)
+        with cgutils.if_unlikely(builder, is_neg):
+            context.call_conv.return_user_exc(builder, ValueError,
+                                              ("negative dimensions not allowed",))
+    return arrtype, shapes
+
+
+def _parse_empty_like_args(context, builder, sig, args):
+    """
+    Parse the arguments of a np.empty_like(), np.zeros_like() or
+    np.ones_like() call.
+    """
+    arytype = sig.args[0]
+    ary = make_array(arytype)(context, builder, value=args[0])
+    shapes = cgutils.unpack_tuple(builder, ary.shape, count=arytype.ndim)
+    return sig.return_type, shapes
+
+
+@builtin
+@implement(numpy.empty, types.Any)
+@implement(numpy.empty, types.Any, types.Any)
+def numpy_empty_nd(context, builder, sig, args):
+    arrtype, shapes = _parse_empty_args(context, builder, sig, args)
+    ary = _empty_nd_impl(context, builder, arrtype, shapes)
+    return ary._getvalue()
+
+@builtin
+@implement(numpy.empty_like, types.Kind(types.Array))
+@implement(numpy.empty_like, types.Kind(types.Array), types.Kind(types.DTypeSpec))
+def numpy_empty_like_nd(context, builder, sig, args):
+    arrtype, shapes = _parse_empty_like_args(context, builder, sig, args)
+    ary = _empty_nd_impl(context, builder, arrtype, shapes)
+    return ary._getvalue()
+
+
+@builtin
+@implement(numpy.zeros, types.Any)
+@implement(numpy.zeros, types.Any, types.Any)
+def numpy_zeros_nd(context, builder, sig, args):
+    arrtype, shapes = _parse_empty_args(context, builder, sig, args)
+    ary = _empty_nd_impl(context, builder, arrtype, shapes)
+    _zero_fill_array(context, builder, ary)
+    return ary._getvalue()
+
+
+@builtin
+@implement(numpy.zeros_like, types.Kind(types.Array))
+@implement(numpy.zeros_like, types.Kind(types.Array), types.Kind(types.DTypeSpec))
+def numpy_zeros_like_nd(context, builder, sig, args):
+    arrtype, shapes = _parse_empty_like_args(context, builder, sig, args)
+    ary = _empty_nd_impl(context, builder, arrtype, shapes)
+    _zero_fill_array(context, builder, ary)
+    return ary._getvalue()
+
+
+if numpy_version >= (1, 8):
+    @builtin
+    @implement(numpy.full, types.Any, types.Any)
+    def numpy_full_nd(context, builder, sig, args):
+
+        def full(shape, value):
+            arr = numpy.empty(shape)
+            for idx in numpy.ndindex(arr.shape):
+                arr[idx] = value
+            return arr
+
+        return context.compile_internal(builder, full, sig, args)
+
+    @builtin
+    @implement(numpy.full, types.Any, types.Any, types.Kind(types.DTypeSpec))
+    def numpy_full_dtype_nd(context, builder, sig, args):
+
+        def full(shape, value, dtype):
+            arr = numpy.empty(shape, dtype)
+            for idx in numpy.ndindex(arr.shape):
+                arr[idx] = value
+            return arr
+
+        return context.compile_internal(builder, full, sig, args)
+
+
+    @builtin
+    @implement(numpy.full_like, types.Kind(types.Array), types.Any)
+    def numpy_full_like_nd(context, builder, sig, args):
+
+        def full_like(arr, value):
+            arr = numpy.empty_like(arr)
+            for idx in numpy.ndindex(arr.shape):
+                arr[idx] = value
+            return arr
+
+        return context.compile_internal(builder, full_like, sig, args)
+
+
+    @builtin
+    @implement(numpy.full_like, types.Kind(types.Array), types.Any, types.Kind(types.DTypeSpec))
+    def numpy_full_like_nd(context, builder, sig, args):
+
+        def full_like(arr, value, dtype):
+            arr = numpy.empty_like(arr, dtype)
+            for idx in numpy.ndindex(arr.shape):
+                arr[idx] = value
+            return arr
+
+        return context.compile_internal(builder, full_like, sig, args)
+
+
+@builtin
+@implement(numpy.ones, types.Any)
+def numpy_ones_nd(context, builder, sig, args):
+
+    def ones(shape):
+        arr = numpy.empty(shape)
+        for idx in numpy.ndindex(arr.shape):
+            arr[idx] = 1
+        return arr
+
+    valty = sig.return_type.dtype
+    return context.compile_internal(builder, ones, sig, args,
+                                    locals={'c': valty})
+
+@builtin
+@implement(numpy.ones, types.Any, types.Kind(types.DTypeSpec))
+def numpy_ones_dtype_nd(context, builder, sig, args):
+
+    def ones(shape, dtype):
+        arr = numpy.empty(shape, dtype)
+        for idx in numpy.ndindex(arr.shape):
+            arr[idx] = 1
+        return arr
+
+    return context.compile_internal(builder, ones, sig, args)
+
+@builtin
+@implement(numpy.ones_like, types.Kind(types.Array))
+def numpy_ones_like_nd(context, builder, sig, args):
+
+    def ones_like(arr):
+        arr = numpy.empty_like(arr)
+        for idx in numpy.ndindex(arr.shape):
+            arr[idx] = 1
+        return arr
+
+    return context.compile_internal(builder, ones_like, sig, args)
+
+@builtin
+@implement(numpy.ones_like, types.Kind(types.Array), types.Kind(types.DTypeSpec))
+def numpy_ones_like_dtype_nd(context, builder, sig, args):
+
+    def ones_like(arr, dtype):
+        arr = numpy.empty_like(arr, dtype)
+        for idx in numpy.ndindex(arr.shape):
+            arr[idx] = 1
+        return arr
+
+    return context.compile_internal(builder, ones_like, sig, args)
+
+
+@builtin
+@implement(numpy.identity, types.Kind(types.Integer))
+def numpy_identity(context, builder, sig, args):
+
+    def identity(n):
+        arr = numpy.zeros((n, n))
+        for i in range(n):
+            arr[i, i] = 1
+        return arr
+
+    return context.compile_internal(builder, identity, sig, args)
+
+@builtin
+@implement(numpy.identity, types.Kind(types.Integer), types.Kind(types.DTypeSpec))
+def numpy_identity(context, builder, sig, args):
+
+    def identity(n, dtype):
+        arr = numpy.zeros((n, n), dtype)
+        for i in range(n):
+            arr[i, i] = 1
+        return arr
+
+    return context.compile_internal(builder, identity, sig, args)
+
+
+@builtin
+@implement(numpy.eye, types.Kind(types.Integer))
+def numpy_eye(context, builder, sig, args):
+
+    def eye(n):
+        return numpy.identity(n)
+
+    return context.compile_internal(builder, eye, sig, args)
+
+@builtin
+@implement(numpy.eye, types.Kind(types.Integer), types.Kind(types.Integer))
+def numpy_eye(context, builder, sig, args):
+
+    def eye(n, m):
+        return numpy.eye(n, m, 0, numpy.float64)
+
+    return context.compile_internal(builder, eye, sig, args)
+
+@builtin
+@implement(numpy.eye, types.Kind(types.Integer), types.Kind(types.Integer),
+           types.Kind(types.Integer))
+def numpy_eye(context, builder, sig, args):
+
+    def eye(n, m, k):
+        return numpy.eye(n, m, k, numpy.float64)
+
+    return context.compile_internal(builder, eye, sig, args)
+
+@builtin
+@implement(numpy.eye, types.Kind(types.Integer), types.Kind(types.Integer),
+           types.Kind(types.Integer), types.Kind(types.DTypeSpec))
+def numpy_eye(context, builder, sig, args):
+
+    def eye(n, m, k, dtype):
+        arr = numpy.zeros((n, m), dtype)
+        if k >= 0:
+            d = min(n, m - k)
+            for i in range(d):
+                arr[i, i + k] = 1
+        else:
+            d = min(n + k, m)
+            for i in range(d):
+                arr[i - k, i] = 1
+        return arr
+
+    return context.compile_internal(builder, eye, sig, args)
+
+
+@builtin
+@implement(numpy.arange, types.Kind(types.Number))
+def numpy_arange_1(context, builder, sig, args):
+    dtype = getattr(numpy, str(sig.return_type.dtype))
+
+    def arange(stop):
+        return numpy.arange(0, stop, 1, dtype)
+
+    return context.compile_internal(builder, arange, sig, args)
+
+@builtin
+@implement(numpy.arange, types.Kind(types.Number), types.Kind(types.Number))
+def numpy_arange_2(context, builder, sig, args):
+    dtype = getattr(numpy, str(sig.return_type.dtype))
+
+    def arange(start, stop):
+        return numpy.arange(start, stop, 1, dtype)
+
+    return context.compile_internal(builder, arange, sig, args)
+
+
+@builtin
+@implement(numpy.arange, types.Kind(types.Number), types.Kind(types.Number),
+           types.Kind(types.Number))
+def numpy_arange_3(context, builder, sig, args):
+    dtype = getattr(numpy, str(sig.return_type.dtype))
+
+    def arange(start, stop, step):
+        return numpy.arange(start, stop, step, dtype)
+
+    return context.compile_internal(builder, arange, sig, args)
+
+@builtin
+@implement(numpy.arange, types.Kind(types.Number), types.Kind(types.Number),
+           types.Kind(types.Number), types.Kind(types.DTypeSpec))
+def numpy_arange_4(context, builder, sig, args):
+
+    if any(isinstance(a, types.Complex) for a in sig.args):
+        def arange(start, stop, step, dtype):
+            nitems_c = (stop - start) / step
+            nitems_r = math.ceil(nitems_c.real)
+            nitems_i = math.ceil(nitems_c.imag)
+            nitems = max(min(nitems_i, nitems_r), 0)
+            arr = numpy.empty(nitems, dtype)
+            val = start
+            for i in range(nitems):
+                arr[i] = val
+                val += step
+            return arr
+    else:
+        def arange(start, stop, step, dtype):
+            nitems_r = math.ceil((stop - start) / step)
+            nitems = max(nitems_r, 0)
+            arr = numpy.empty(nitems, dtype)
+            val = start
+            for i in range(nitems):
+                arr[i] = val
+                val += step
+            return arr
+
+    return context.compile_internal(builder, arange, sig, args,
+                                    locals={'nitems': types.intp})
+
+
+@builtin
+@implement(numpy.linspace, types.Kind(types.Number), types.Kind(types.Number))
+def numpy_linspace_2(context, builder, sig, args):
+
+    def linspace(start, stop):
+        return numpy.linspace(start, stop, 50)
+
+    return context.compile_internal(builder, linspace, sig, args)
+
+@builtin
+@implement(numpy.linspace, types.Kind(types.Number), types.Kind(types.Number),
+           types.Kind(types.Integer))
+def numpy_linspace_3(context, builder, sig, args):
+    dtype = getattr(numpy, str(sig.return_type.dtype))
+
+    def linspace(start, stop, num):
+        arr = numpy.empty(num, dtype)
+        div = num - 1
+        delta = stop - start
+        arr[0] = start
+        for i in range(1, num):
+            arr[i] = start + delta * (i / div)
+        return arr
+
+    return context.compile_internal(builder, linspace, sig, args)
+
