@@ -146,6 +146,12 @@ class BaseContext(object):
     implement_powi_as_math_call = False
     implement_pow_as_math_call = False
 
+    # Bound checking
+    enable_boundcheck = False
+
+    # NRT
+    enable_nrt = False
+
     def __init__(self, typing_context):
         _load_global_helpers()
         self.address_size = utils.MACHINE_BITS
@@ -154,6 +160,7 @@ class BaseContext(object):
         self.defns = defaultdict(Overloads)
         self.attrs = defaultdict(Overloads)
         self.generators = {}
+        self.special_ops = {}
 
         self.install_registry(builtin_registry)
 
@@ -177,6 +184,14 @@ class BaseContext(object):
     def target_data(self):
         raise NotImplementedError
 
+    def subtarget(self, **kws):
+        obj = copy.copy(self)  # shallow copy
+        for k, v in kws.items():
+            if not hasattr(obj, k):
+                raise NameError("unknown option {0!r}".format(k))
+            setattr(obj, k, v)
+        return obj
+
     def install_registry(self, registry):
         """
         Install a *registry* (a imputils.Registry instance) of function
@@ -315,7 +330,13 @@ class BaseContext(object):
         Return a LLVM constant representing value *val* of Numba type *ty*.
         """
         if self.is_struct_type(ty):
-            return self.get_constant_struct(builder, ty, val)
+            struct = self.get_constant_struct(builder, ty, val)
+            if isinstance(ty, types.Record):
+                ptrty = self.data_model_manager[ty].get_data_type()
+                ptr = cgutils.alloca_once(builder, ptrty)
+                builder.store(struct, ptr)
+                return ptr
+            return struct
 
         elif isinstance(ty, types.ExternalFunctionPointer):
             ptrty = self.get_function_pointer_type(ty)
@@ -413,7 +434,7 @@ class BaseContext(object):
         Return the implementation of function *fn* for signature *sig*.
         The return value is a callable with the signature (builder, args).
         """
-        if isinstance(fn, types.Function):
+        if isinstance(fn, (types.NumberClass, types.Function)):
             key = fn.template.key
 
             if isinstance(key, MethodType):
@@ -477,9 +498,10 @@ class BaseContext(object):
                         shape=cgutils.pack_array(builder, newshape),
                         strides=cgutils.pack_array(builder, newstrides),
                         itemsize=context.get_constant(types.intp, elemty.size),
-                        parent=ary.parent
+                        meminfo=None,
+                        parent=None,
                     )
-                    
+
                     return ary._getvalue()
             else:
                 @impl_attribute(typ, attr, elemty)
@@ -494,17 +516,15 @@ class BaseContext(object):
             # We are treating them as constants.
             # XXX We shouldn't have to retype this
             attrty = self.typing_context.resolve_module_constants(typ, attr)
-            if attrty is not None and not isinstance(attrty, (types.Dispatcher,
-                                                              types.Function,
-                                                              types.Module)):
+            if attrty is not None and not isinstance(attrty, types.Dummy):
                 pyval = getattr(typ.pymod, attr)
                 llval = self.get_constant(attrty, pyval)
                 @impl_attribute(typ, attr, attrty)
                 def imp(context, builder, typ, val):
                     return llval
                 return imp
-            # No implementation required for functions/modules, which are
-            # dealt with later
+            # No implementation required for dummies (functions, modules...),
+            # which are dealt with later
             return None
 
         # Lookup specific attribute implementation for this type
@@ -706,6 +726,9 @@ class BaseContext(object):
             assert toty.layout == 'A'
             return val
 
+        elif fromty in types.integer_domain and toty == types.voidptr:
+            return builder.inttoptr(val, self.get_value_type(toty))
+
         raise NotImplementedError("cast", val, fromty, toty)
 
     def make_optional(self, optionaltype):
@@ -809,6 +832,29 @@ class BaseContext(object):
     def get_dummy_type(self):
         return GENERIC_POINTER
 
+    def compile_only_no_cache(self, builder, impl, sig, locals={}):
+        """Invoke the compiler to compile a function to be used inside a
+        nopython function, but without generating code to call that
+        function.
+        """
+        # Compile
+        from numba import compiler
+
+        codegen = self.jit_codegen()
+        library = codegen.create_library(impl.__name__)
+        flags = compiler.Flags()
+        flags.set('no_compile')
+        flags.set('no_cpython_wrapper')
+        cres = compiler.compile_internal(self.typing_context, self,
+                                         library,
+                                         impl, sig.args,
+                                         sig.return_type, flags,
+                                         locals=locals)
+
+        # Allow inlining the function inside callers.
+        codegen.add_linking_library(cres.library)
+        return cres
+
     def compile_internal(self, builder, impl, sig, args, locals={}):
         """Invoke compiler to implement a function for a nopython function
         """
@@ -820,25 +866,17 @@ class BaseContext(object):
         fndesc = self.cached_internal_func.get(cache_key)
 
         if fndesc is None:
-            # Compile
-            from numba import compiler
-
-            codegen = self.jit_codegen()
-            library = codegen.create_library(impl.__name__)
-            flags = compiler.Flags()
-            flags.set('no_compile')
-            flags.set('no_cpython_wrapper')
-            cres = compiler.compile_internal(self.typing_context, self,
-                                             library,
-                                             impl, sig.args,
-                                             sig.return_type, flags,
-                                             locals=locals)
-
-            # Allow inlining the function inside callers.
-            codegen.add_linking_library(cres.library)
+            cres = self.compile_only_no_cache(builder, impl, sig,
+                                              locals=locals)
             fndesc = cres.fndesc
             self.cached_internal_func[cache_key] = fndesc
 
+        return self.call_internal(builder, fndesc, sig, args)
+
+    def call_internal(self, builder, fndesc, sig, args):
+        """Given the function descriptor of an internally compiled function,
+        emit a call to that function with the given arguments.
+        """
         # Add call to the generated function
         llvm_mod = cgutils.get_module(builder)
         fn = self.declare_function(llvm_mod, fndesc)
@@ -916,9 +954,19 @@ class BaseContext(object):
 
         # Create array structure
         cary = self.make_array(typ)(self, builder)
-        cary.data = builder.bitcast(data, cary.data.type)
-        cary.shape = cshape
-        cary.strides = cstrides
+
+        rt_addr = self.get_constant(types.uintp, id(ary)).inttoptr(
+            self.get_value_type(types.pyobject))
+
+        intp_itemsize = self.get_constant(types.intp, ary.dtype.itemsize)
+        self.populate_array(cary,
+                            data=builder.bitcast(data, cary.data.type),
+                            shape=cshape,
+                            strides=cstrides,
+                            itemsize=intp_itemsize,
+                            parent=rt_addr,
+                            meminfo=None)
+
         return cary._getvalue()
 
     def get_abi_sizeof(self, ty):
@@ -947,6 +995,60 @@ class BaseContext(object):
         """
         return lc.Module.new(name)
 
+    def nrt_meminfo_alloc(self, builder, size):
+        if not self.enable_nrt:
+            raise Exception("Require NRT")
+        mod = cgutils.get_module(builder)
+        fnty = llvmir.FunctionType(llvmir.IntType(8).as_pointer(),
+            [self.get_value_type(types.intp)])
+        fn = mod.get_or_insert_function(fnty, name="NRT_MemInfo_alloc_safe")
+        return builder.call(fn, [size])
+
+    def nrt_meminfo_data(self, builder, meminfo):
+        if not self.enable_nrt:
+            raise Exception("Require NRT")
+        mod = cgutils.get_module(builder)
+        voidptr = llvmir.IntType(8).as_pointer()
+        fnty = llvmir.FunctionType(voidptr, [voidptr])
+        fn = mod.get_or_insert_function(fnty, name="NRT_MemInfo_data")
+        return builder.call(fn, [meminfo])
+
+    def get_nrt_meminfo(self, builder, typ, value):
+        return self.data_model_manager[typ].get_nrt_meminfo(builder, value)
+
+    def nrt_incref(self, builder, typ, value):
+        if not self.enable_nrt:
+            raise Exception("Require NRT")
+
+        members = self.data_model_manager[typ].traverse(builder, value)
+        for mt, mv in members:
+            self.nrt_incref(builder, mt, mv)
+
+        meminfo = self.get_nrt_meminfo(builder, typ, value)
+        if meminfo:
+            mod = cgutils.get_module(builder)
+            fnty = llvmir.FunctionType(llvmir.VoidType(),
+                [llvmir.IntType(8).as_pointer()])
+            fn = mod.get_or_insert_function(fnty, name="NRT_incref")
+
+            builder.call(fn, [meminfo])
+
+    def nrt_decref(self, builder, typ, value):
+        if not self.enable_nrt:
+            raise Exception("Require NRT")
+
+        members = self.data_model_manager[typ].traverse(builder, value)
+        for mt, mv in members:
+            self.nrt_decref(builder, mt, mv)
+
+        meminfo = self.get_nrt_meminfo(builder, typ, value)
+        if meminfo:
+            mod = cgutils.get_module(builder)
+            fnty = llvmir.FunctionType(llvmir.VoidType(),
+                [llvmir.IntType(8).as_pointer()])
+            fn = mod.get_or_insert_function(fnty, name="NRT_decref")
+            builder.call(fn, [meminfo])
+
 
 class _wrap_impl(object):
     def __init__(self, imp, context, sig):
