@@ -926,6 +926,8 @@ def complex_div_impl(context, builder, sig, args):
         aimag = a.imag
         breal = b.real
         bimag = b.imag
+        if not breal and not bimag:
+            raise ZeroDivisionError("complex division by zero")
         if abs(breal) >= abs(bimag):
             # Divide tops and bottom by b.real
             if not breal:
@@ -1213,6 +1215,24 @@ def getitem_unituple(context, builder, sig, args):
     builder.position_at_end(bbend)
     return phinode
 
+
+@builtin
+@implement('getitem', types.Kind(types.CPointer), types.Kind(types.Integer))
+def getitem_cpointer(context, builder, sig, args):
+    base_ptr, idx = args
+    elem_ptr = builder.gep(base_ptr, [idx])
+    return builder.load(elem_ptr)
+
+
+@builtin
+@implement('setitem', types.Kind(types.CPointer), types.Kind(types.Integer),
+           types.Any)
+def setitem_cpointer(context, builder, sig, args):
+    base_ptr, idx, val = args
+    elem_ptr = builder.gep(base_ptr, [idx])
+    return builder.store(val, elem_ptr)
+
+
 #-------------------------------------------------------------------------------
 
 
@@ -1225,12 +1245,29 @@ def caster(restype):
 
     return _cast
 
-for tp in types.number_domain:
+cast_types = set(types.number_domain)
+cast_types.add(types.bool_)
+for tp in cast_types:
     builtin(caster(tp))
 
 
 #-------------------------------------------------------------------------------
 
+def generic_compare(context, builder, key, argtypes, args):
+    """
+    Compare the given LLVM values of the given Numba types using
+    the comparison *key* (e.g. '==').  The values are first cast to
+    a common safe conversion type.
+    """
+    at, bt = argtypes
+    av, bv = args
+    ty = context.typing_context.unify_types(at, bt)
+    cav = context.cast(builder, av, at, ty)
+    cbv = context.cast(builder, bv, bt, ty)
+    cmpsig = typing.signature(types.boolean, ty, ty)
+    cmpfunc = context.get_function(key, cmpsig)
+    return cmpfunc(builder, (cav, cbv))
+
 @builtin
 @implement(max, types.VarArg(types.Any))
 def max_impl(context, builder, sig, args):
@@ -1429,15 +1466,69 @@ def array_ravel_impl(context, builder, sig, args):
 # -----------------------------------------------------------------------------
 
 @builtin
-@implement(types.len_type, types.Kind(types.Tuple))
-def array_len(context, builder, sig, args):
+@implement(types.len_type, types.Kind(types.BaseTuple))
+def tuple_len(context, builder, sig, args):
     tupty, = sig.args
     retty = sig.return_type
     return context.get_constant(retty, len(tupty.types))
 
+def tuple_cmp_ordered(context, builder, op, sig, args):
+    tu, tv = sig.args
+    u, v = args
+    res = cgutils.alloca_once_value(builder, cgutils.true_bit)
+    bbend = cgutils.append_basic_block(builder, "cmp_end")
+    for i, (ta, tb) in enumerate(zip(tu.types, tv.types)):
+        a = builder.extract_value(u, i)
+        b = builder.extract_value(v, i)
+        not_equal = generic_compare(context, builder, '!=', (ta, tb), (a, b))
+        with cgutils.ifthen(builder, not_equal):
+            pred = generic_compare(context, builder, op, (ta, tb), (a, b))
+            builder.store(pred, res)
+            builder.branch(bbend)
+    # Everything matched equal => compare lengths
+    len_compare = eval("%d %s %d" % (len(tu.types), op, len(tv.types)))
+    pred = context.get_constant(types.boolean, len_compare)
+    builder.store(pred, res)
+    builder.branch(bbend)
+    builder.position_at_end(bbend)
+    return builder.load(res)
+
 @builtin
-@implement(types.len_type, types.Kind(types.UniTuple))
-def array_len(context, builder, sig, args):
-    tupty, = sig.args
-    retty = sig.return_type
-    return context.get_constant(retty, tupty.count)
+@implement('==', types.Kind(types.BaseTuple), types.Kind(types.BaseTuple))
+def tuple_eq(context, builder, sig, args):
+    tu, tv = sig.args
+    u, v = args
+    if len(tu.types) != len(tv.types):
+        return context.get_constant(types.boolean, False)
+    res = context.get_constant(types.boolean, True)
+    for i, (ta, tb) in enumerate(zip(tu.types, tv.types)):
+        a = builder.extract_value(u, i)
+        b = builder.extract_value(v, i)
+        pred = generic_compare(context, builder, "==", (ta, tb), (a, b))
+        res = builder.and_(res, pred)
+    return res
+
+@builtin
+@implement('!=', types.Kind(types.BaseTuple), types.Kind(types.BaseTuple))
+def tuple_ne(context, builder, sig, args):
+    return builder.not_(tuple_eq(context, builder, sig, args))
+
+@builtin
+@implement('<', types.Kind(types.BaseTuple), types.Kind(types.BaseTuple))
+def tuple_lt(context, builder, sig, args):
+    return tuple_cmp_ordered(context, builder, '<', sig, args)
+
+@builtin
+@implement('<=', types.Kind(types.BaseTuple), types.Kind(types.BaseTuple))
+def tuple_le(context, builder, sig, args):
+    return tuple_cmp_ordered(context, builder, '<=', sig, args)
+
+@builtin
+@implement('>', types.Kind(types.BaseTuple), types.Kind(types.BaseTuple))
+def tuple_gt(context, builder, sig, args):
+    return tuple_cmp_ordered(context, builder, '>', sig, args)
+
+@builtin
+@implement('>=', types.Kind(types.BaseTuple), types.Kind(types.BaseTuple))
+def tuple_ge(context, builder, sig, args):
+    return tuple_cmp_ordered(context, builder, '>=', sig, args)
