@@ -15,7 +15,8 @@ from numba.targets import (
     callconv, codegen, externals, intrinsics, cmathimpl, mathimpl,
     npyimpl, operatorimpl, printimpl, randomimpl)
 from .options import TargetOptions
-
+from numba.runtime.atomicops import install_atomic_refct
+from numba.runtime import rtsys
 
 # Keep those structures in sync with _dynfunc.c.
 
@@ -55,6 +56,9 @@ class CPUContext(BaseContext):
 
         self._internal_codegen = codegen.JITCPUCodegen("numba.exec")
 
+        # Initialize NRT runtime
+        rtsys.initialize(self)
+
     @property
     def target_data(self):
         return self._internal_codegen.target_data
@@ -109,6 +113,8 @@ class CPUContext(BaseContext):
             # calls to compiler-rt
             intrinsics.fix_divmod(mod)
 
+        install_atomic_refct(mod)
+
     def create_cpython_wrapper(self, library, fndesc, call_helper,
                                release_gil=False):
         wrapper_module = self.create_module("wrapper")
@@ -165,6 +171,8 @@ class CPUTargetOptions(TargetOptions):
         "looplift": bool,
         "wraparound": bool,
         "boundcheck": bool,
+        "_nrt": bool,
+        "no_rewrites": bool,
     }
 
 
