@@ -12,39 +12,17 @@ from collections import namedtuple
 
 from llvmlite.llvmpy import core as lc
 
-from . import builtins, ufunc_db
+from . import builtins, ufunc_db, arrayobj
 from .imputils import implement, Registry
 from .. import typing, types, cgutils, numpy_support
 from ..config import PYVERSION
 from ..numpy_support import ufunc_find_matching_loop
+from ..typing import npydecl
 
 registry = Registry()
 register = registry.register
 
 
-def caster(restype, constructor):
-    """
-    Implement explicit calls to Numpy type *constructor* (e.g. np.int16).
-    """
-    @implement(constructor, types.Any)
-    def _cast(context, builder, sig, args):
-        [val] = args
-        [valty] = sig.args
-        return context.cast(builder, val, valty, restype)
-
-    return _cast
-
-def register_casters(register_function):
-    for tp in types.number_domain:
-        register_function(caster(tp, getattr(numpy, str(tp))))
-
-    register_function(caster(types.intc, numpy.intc))
-    register_function(caster(types.uintc, numpy.uintc))
-    register_function(caster(types.intp, numpy.intp))
-    register_function(caster(types.uintp, numpy.uintp))
-
-register_casters(register)
-
 ########################################################################
 
 # In the way we generate code, ufuncs work with scalar as well as
@@ -190,6 +168,99 @@ def _prepare_argument(ctxt, bld, inp, tyinp, where='input operand'):
         raise TypeError('unknown type for {0}: {1}'.format(where, str(tyinp)))
 
 
+_broadcast_onto_sig = types.intp(types.intp, types.CPointer(types.intp),
+                                 types.intp, types.CPointer(types.intp))
+def _broadcast_onto(src_ndim, src_shape, dest_ndim, dest_shape):
+    '''Low-level utility function used in calculating a shape for
+    an implicit output array.  This function assumes that the
+    destination shape is an LLVM pointer to a C-style array that was
+    already initialized to a size of one along all axes.
+
+    Returns an integer value:
+    >= 1  :  Succeeded.  Return value should equal the number of dimensions in
+             the destination shape.
+    0     :  Failed to broadcast because source shape is larger than the
+             destination shape (this case should be weeded out at type
+             checking).
+    < 0   :  Failed to broadcast onto destination axis, at axis number ==
+             -(return_value + 1).
+    '''
+    if src_ndim > dest_ndim:
+        # This check should have been done during type checking, but
+        # let's be defensive anyway...
+        return 0
+    else:
+        src_index = 0
+        dest_index = dest_ndim - src_ndim
+        while src_index < src_ndim:
+            src_dim_size = src_shape[src_index]
+            dest_dim_size = dest_shape[dest_index]
+            # Check to see if we've already mutated the destination
+            # shape along this axis.
+            if dest_dim_size != 1:
+                # If we have mutated the destination shape already,
+                # then the source axis size must either be one,
+                # or the destination axis size.
+                if src_dim_size != dest_dim_size and src_dim_size != 1:
+                    return -(dest_index + 1)
+            elif src_dim_size != 1:
+                # If the destination size is still its initial
+                dest_shape[dest_index] = src_dim_size
+            src_index += 1
+            dest_index += 1
+    return dest_index
+
+def _build_array(context, builder, array_ty, arg_arrays):
+    """Utility function to handle allocation of an implicit output array
+    given the target context, builder, output array type, and a list of
+    _ArrayHelper instances.
+    """
+    intp_ty = context.get_value_type(types.intp)
+    def make_intp_const(val):
+        return context.get_constant(types.intp, val)
+
+    ZERO = make_intp_const(0)
+    ONE = make_intp_const(1)
+
+    src_shape = cgutils.alloca_once(builder, intp_ty, array_ty.ndim,
+                                    "src_shape")
+    dest_ndim = make_intp_const(array_ty.ndim)
+    dest_shape = cgutils.alloca_once(builder, intp_ty, array_ty.ndim,
+                                     "dest_shape")
+    dest_shape_addrs = tuple(builder.gep(dest_shape, [make_intp_const(index)])
+                           for index in range(array_ty.ndim))
+
+    # Initialize the destination shape with all ones.
+    for dest_shape_addr in dest_shape_addrs:
+        builder.store(ONE, dest_shape_addr)
+
+    # For each argument, try to broadcast onto the destination shape,
+    # mutating along any axis where the argument shape is not one and
+    # the destination shape is one.
+    for arg_number, arg in enumerate(arg_arrays):
+        if not hasattr(arg, "ndim"): # Skip scalar arguments
+            continue
+        arg_ndim = make_intp_const(arg.ndim)
+        for index in range(arg.ndim):
+            builder.store(builder.extract_value(arg.ary.shape, index),
+                          builder.gep(src_shape, [make_intp_const(index)]))
+        arg_result = context.compile_internal(
+            builder, _broadcast_onto, _broadcast_onto_sig,
+            [arg_ndim, src_shape, dest_ndim, dest_shape])
+        with cgutils.if_unlikely(builder,
+                                 builder.icmp(lc.ICMP_SLT, arg_result, ONE)):
+            msg = "unable to broadcast argument %d to output array" % (
+                arg_number,)
+            context.call_conv.return_user_exc(builder, ValueError, (msg,))
+
+    dest_shape_tup = tuple(builder.load(dest_shape_addr)
+                           for dest_shape_addr in dest_shape_addrs)
+    array_val = arrayobj._empty_nd_impl(context, builder, array_ty,
+                                        dest_shape_tup)
+    return _prepare_argument(context, builder, array_val._getvalue(), array_ty,
+                             where='implicit output argument')
+
+
 def numpy_ufunc_kernel(context, builder, sig, args, kernel_class,
                        explicit_output=True):
     # This is the code generator that builds all the looping needed
@@ -203,13 +274,18 @@ def numpy_ufunc_kernel(context, builder, sig, args, kernel_class,
     # kernel_class -  a code generating subclass of _Kernel that provides
     # explicit_output - if the output was explicit in the call
     #                   (ie: np.add(x,y,r))
-    if not explicit_output:
-        args.append(lc.Constant.null(context.get_value_type(sig.return_type)))
-        tyargs = sig.args + (sig.return_type,)
-    else:
-        tyargs = sig.args
+
     arguments = [_prepare_argument(context, builder, arg, tyarg)
-                 for arg, tyarg in zip(args, tyargs)]
+                 for arg, tyarg in zip(args, sig.args)]
+    if not explicit_output:
+        ret_ty = sig.return_type
+        if isinstance(ret_ty, types.Array):
+            output = _build_array(context, builder, ret_ty, arguments)
+        else:
+            output = _prepare_argument(
+                context, builder,
+                lc.Constant.null(context.get_value_type(ret_ty)), ret_ty)
+        arguments.append(output)
 
     inputs = arguments[0:-1]
     output = arguments[-1]
@@ -313,11 +389,13 @@ def _ufunc_db_function(ufunc):
 ################################################################################
 # Helper functions that register the ufuncs
 
+_kernels = {} # Temporary map from ufunc's to their kernel implementation class
+
 def register_unary_ufunc_kernel(ufunc, kernel):
     def unary_ufunc(context, builder, sig, args):
         return numpy_ufunc_kernel(context, builder, sig, args, kernel)
 
-    def unary_scalar_ufunc(context, builder, sig, args):
+    def unary_ufunc_no_explicit_output(context, builder, sig, args):
         return numpy_ufunc_kernel(context, builder, sig, args, kernel,
                                   explicit_output=False)
 
@@ -325,15 +403,17 @@ def register_unary_ufunc_kernel(ufunc, kernel):
 
     # (array or scalar, out=array)
     register(implement(ufunc, _any, types.Kind(types.Array))(unary_ufunc))
-    # (scalar, out=array)
-    register(implement(ufunc, _any)(unary_scalar_ufunc))
+    # (array or scalar)
+    register(implement(ufunc, _any)(unary_ufunc_no_explicit_output))
+
+    _kernels[ufunc] = kernel
 
 
 def register_binary_ufunc_kernel(ufunc, kernel):
     def binary_ufunc(context, builder, sig, args):
         return numpy_ufunc_kernel(context, builder, sig, args, kernel)
 
-    def binary_scalar_ufunc(context, builder, sig, args):
+    def binary_ufunc_no_explicit_output(context, builder, sig, args):
         return numpy_ufunc_kernel(context, builder, sig, args, kernel,
                                   explicit_output=False)
 
@@ -342,7 +422,28 @@ def register_binary_ufunc_kernel(ufunc, kernel):
     # (array or scalar, array o scalar, out=array)
     register(implement(ufunc, _any, _any, types.Kind(types.Array))(binary_ufunc))
     # (scalar, scalar)
-    register(implement(ufunc, _any, _any)(binary_scalar_ufunc))
+    register(implement(ufunc, _any, _any)(binary_ufunc_no_explicit_output))
+
+    _kernels[ufunc] = kernel
+
+
+def register_unary_operator_kernel(operator, kernel):
+    def lower_unary_operator(context, builder, sig, args):
+        return numpy_ufunc_kernel(context, builder, sig, args, kernel,
+                                  explicit_output=False)
+    _arr_kind = types.Kind(types.Array)
+    register(implement(operator, _arr_kind)(lower_unary_operator))
+
+
+def register_binary_operator_kernel(operator, kernel):
+    def lower_binary_operator(context, builder, sig, args):
+        return numpy_ufunc_kernel(context, builder, sig, args, kernel,
+                                  explicit_output=False)
+    _any = types.Any
+    _arr_kind = types.Kind(types.Array)
+    register(implement(operator, _arr_kind, _arr_kind)(lower_binary_operator))
+    register(implement(operator, _any, _arr_kind)(lower_binary_operator))
+    register(implement(operator, _arr_kind, _any)(lower_binary_operator))
 
 
 ################################################################################
@@ -356,3 +457,34 @@ for ufunc in ufunc_db.get_ufuncs():
     else:
         raise RuntimeError("Don't know how to register ufuncs from ufunc_db with arity > 2")
 
+
+@register
+@implement('+', types.Kind(types.Array))
+def array_positive_impl(context, builder, sig, args):
+    '''Lowering function for +(array) expressions.  Defined here
+    (numba.targets.npyimpl) since the remaining array-operator
+    lowering functions are also registered in this module.
+    '''
+    class _UnaryPositiveKernel(_Kernel):
+        def generate(self, *args):
+            [val] = args
+            return val
+
+    return numpy_ufunc_kernel(context, builder, sig, args,
+                              _UnaryPositiveKernel, explicit_output=False)
+
+
+for _op_map in (npydecl.NumpyRulesUnaryArrayOperator._op_map,
+                npydecl.NumpyRulesArrayOperator._op_map):
+    for operator, ufunc_name in _op_map.items():
+        ufunc = getattr(numpy, ufunc_name)
+        kernel = _kernels[ufunc]
+        if ufunc.nin == 1:
+            register_unary_operator_kernel(operator, kernel)
+        elif ufunc.nin == 2:
+            register_binary_operator_kernel(operator, kernel)
+        else:
+            raise RuntimeError("There shouldn't be any non-unary or binary operators")
+
+
+del _kernels
