@@ -26,7 +26,8 @@ force_pyobj_flags.set("force_pyobject")
 no_pyobj_flags = Flags()
 
 
-skip_on_numpy_16 = unittest.skipIf(np.__version__.startswith("1.6."),
+is_on_numpy_16 = np.__version__.startswith("1.6.")
+skip_on_numpy_16 = unittest.skipIf(is_on_numpy_16,
                                    "test requires Numpy 1.7 or later")
 
 
@@ -101,6 +102,8 @@ class TestCase(unittest.TestCase):
         that the object in question belongs to.  Possible return values
         are: "exact", "complex", "approximate", "sequence", and "unknown"
         """
+        if isinstance(numeric_object, np.ndarray):
+            return "ndarray"
 
         for tp in self._sequence_typesets:
             if isinstance(numeric_object, tp):
@@ -120,6 +123,18 @@ class TestCase(unittest.TestCase):
 
         return "unknown"
 
+    def _fix_dtype(self, dtype):
+        """
+        Fix the given *dtype* for comparison.
+        """
+        # Under 64-bit Windows, Numpy may return either int32 or int64
+        # arrays depending on the function.
+        if (sys.platform == 'win32' and sys.maxsize > 2**32 and
+            dtype == np.dtype('int32')):
+            return np.dtype('int64')
+        else:
+            return dtype
+
     def assertPreciseEqual(self, first, second, prec='exact', ulps=1,
                            msg=None):
         """
@@ -178,6 +193,21 @@ class TestCase(unittest.TestCase):
         compare_family = first_family
 
         # For recognized sequences, recurse
+        if compare_family == "ndarray":
+            dtype = self._fix_dtype(first.dtype)
+            self.assertEqual(dtype, self._fix_dtype(second.dtype))
+            self.assertEqual(first.ndim, second.ndim)
+            self.assertEqual(first.shape, second.shape)
+            self.assertEqual([x / first.itemsize for x in first.strides],
+                             [x / second.itemsize for x in second.strides])
+            if first.dtype != dtype:
+                first = first.astype(dtype)
+            if second.dtype != dtype:
+                second = second.astype(dtype)
+            for a, b in zip(first.flat, second.flat):
+                self._assertPreciseEqual(a, b, prec, ulps, msg)
+            return
+
         if compare_family == "sequence":
             self.assertEqual(len(first), len(second), msg=msg)
             for a, b in zip(first, second):
