@@ -7,6 +7,7 @@ import numpy as np
 from numba import unittest_support as unittest
 from numba import typeof, types
 from numba.compiler import compile_isolated
+from numba.numpy_support import as_dtype
 from .support import TestCase
 
 
@@ -42,6 +43,62 @@ def np_ndindex_array(arr):
     return s
 
 
+def np_around_array(arr, decimals, out):
+    np.around(arr, decimals, out)
+
+def np_around_binary(val, decimals):
+    return np.around(val, decimals)
+
+def np_around_unary(val):
+    return np.around(val)
+
+def np_round_array(arr, decimals, out):
+    np.round(arr, decimals, out)
+
+def np_round_binary(val, decimals):
+    return np.round(val, decimals)
+
+def np_round_unary(val):
+    return np.round(val)
+
+def _fixed_np_round(arr, decimals=0, out=None):
+    """
+    A slightly bugfixed version of np.round().
+    """
+    if out is not None and arr.dtype.kind == 'c':
+        # workaround for https://github.com/numpy/numpy/issues/5779
+        _fixed_np_round(arr.real, decimals, out.real)
+        _fixed_np_round(arr.imag, decimals, out.imag)
+        return out
+    else:
+        res = np.round(arr, decimals, out)
+        if out is None:
+            # workaround for https://github.com/numpy/numpy/issues/5780
+            def fixup_signed_zero(arg, res):
+                if res == 0.0 and arg < 0:
+                    return -np.abs(res)
+                else:
+                    return res
+            if isinstance(arr, (complex, np.complexfloating)):
+                res = complex(fixup_signed_zero(arr.real, res.real),
+                              fixup_signed_zero(arr.imag, res.imag))
+            else:
+                res = fixup_signed_zero(arr, res)
+        return res
+
+
+def array_cumprod(arr):
+    return arr.cumprod()
+
+def array_cumprod_global(arr):
+    return np.cumprod(arr)
+
+def array_cumsum(arr):
+    return arr.cumsum()
+
+def array_cumsum_global(arr):
+    return np.cumsum(arr)
+
 def array_sum(arr):
     return arr.sum()
 
@@ -352,6 +409,23 @@ class TestArrayMethods(TestCase):
 
         np.testing.assert_allclose(np.prod(arr), cfunc(arr))
 
+    def check_cumulative(self, pyfunc):
+        arr = np.arange(2, 10, dtype=np.int16)
+        expected, got = run_comparative(pyfunc, arr)
+        self.assertPreciseEqual(got, expected)
+        arr = np.linspace(2, 8, 6)
+        expected, got = run_comparative(pyfunc, arr)
+        self.assertPreciseEqual(got, expected)
+        arr = arr.reshape((3, 2))
+        expected, got = run_comparative(pyfunc, arr)
+        self.assertPreciseEqual(got, expected)
+
+    def test_array_cumsum(self):
+        self.check_cumulative(array_cumsum)
+
+    def test_array_cumsum_global(self):
+        self.check_cumulative(array_cumsum_global)
+
     def check_aggregation_magnitude(self, pyfunc, is_prod=False):
         """
         Check that integer overflows are avoided (issue #931).
@@ -375,10 +449,18 @@ class TestArrayMethods(TestCase):
         self.check_aggregation_magnitude(array_sum)
         self.check_aggregation_magnitude(array_sum_global)
 
+    def test_cumsum_magnitude(self):
+        self.check_aggregation_magnitude(array_cumsum)
+        self.check_aggregation_magnitude(array_cumsum_global)
+
     def test_prod_magnitude(self):
         self.check_aggregation_magnitude(array_prod, is_prod=True)
         self.check_aggregation_magnitude(array_prod_global, is_prod=True)
 
+    def test_cumprod_magnitude(self):
+        self.check_aggregation_magnitude(array_cumprod, is_prod=True)
+        self.check_aggregation_magnitude(array_cumprod_global, is_prod=True)
+
     def test_mean_magnitude(self):
         self.check_aggregation_magnitude(array_mean)
         self.check_aggregation_magnitude(array_mean_global)
@@ -391,6 +473,87 @@ class TestArrayMethods(TestCase):
         self.check_aggregation_magnitude(array_std)
         self.check_aggregation_magnitude(array_std_global)
 
+    def check_round_array(self, pyfunc):
+        def check_round(cfunc, values, inty, outty, decimals):
+            # Create input and output arrays of the right type
+            arr = values.astype(as_dtype(inty))
+            out = np.zeros_like(arr).astype(as_dtype(outty))
+            pyout = out.copy()
+            _fixed_np_round(arr, decimals, pyout)
+            cfunc(arr, decimals, out)
+            np.testing.assert_allclose(out, pyout)
+            # Output shape mismatch
+            with self.assertRaises(ValueError) as raises:
+                cfunc(arr, decimals, out[1:])
+            self.assertEqual(str(raises.exception),
+                             "invalid output shape")
+
+        def check_types(argtypes, outtypes, values):
+            for inty, outty in product(argtypes, outtypes):
+                cres = compile_isolated(pyfunc,
+                                        (types.Array(inty, 1, 'A'),
+                                         types.int32,
+                                         types.Array(outty, 1, 'A')))
+                cfunc = cres.entry_point
+                check_round(cres.entry_point, values, inty, outty, 0)
+                check_round(cres.entry_point, values, inty, outty, 1)
+                if not isinstance(outty, types.Integer):
+                    check_round(cres.entry_point, values * 10, inty, outty, -1)
+                else:
+                    # Avoid Numpy bug when output is an int:
+                    # https://github.com/numpy/numpy/issues/5777
+                    pass
+
+        values = np.array([-3.0, -2.5, -2.25, -1.5, 1.5, 2.25, 2.5, 2.75])
+
+        argtypes = (types.float64, types.float32, types.int32)
+        check_types(argtypes, argtypes, values)
+
+        argtypes = (types.complex64, types.complex128)
+        check_types(argtypes, argtypes, values * (1 - 1j))
+
+    def test_round_array(self):
+        self.check_round_array(np_round_array)
+
+    def test_around_array(self):
+        self.check_round_array(np_around_array)
+
+    def check_round_scalar(self, unary_pyfunc, binary_pyfunc):
+        base_values = [-3.0, -2.5, -2.25, -1.5, 1.5, 2.25, 2.5, 2.75]
+        complex_values = [x * (1 - 1j) for x in base_values]
+        int_values = [int(x) for x in base_values]
+        argtypes = (types.float64, types.float32, types.int32,
+                    types.complex64, types.complex128)
+        argvalues = [base_values, base_values, int_values,
+                     complex_values, complex_values]
+
+        pyfunc = binary_pyfunc
+        for ty, values in zip(argtypes, argvalues):
+            cres = compile_isolated(pyfunc, (ty, types.int32))
+            cfunc = cres.entry_point
+            for decimals in (1, 0, -1):
+                for v in values:
+                    if decimals > 0:
+                        v *= 10
+                    expected = _fixed_np_round(v, decimals)
+                    got = cfunc(v, decimals)
+                    self.assertPreciseEqual(got, expected)
+
+        pyfunc = unary_pyfunc
+        for ty, values in zip(argtypes, argvalues):
+            cres = compile_isolated(pyfunc, (ty,))
+            cfunc = cres.entry_point
+            for v in values:
+                expected = _fixed_np_round(v)
+                got = cfunc(v)
+                self.assertPreciseEqual(got, expected)
+
+    def test_round_scalar(self):
+        self.check_round_scalar(np_round_unary, np_round_binary)
+
+    def test_around_scalar(self):
+        self.check_round_scalar(np_around_unary, np_around_binary)
+
 
 # These form a testing product where each of the combinations are tested
 reduction_funcs = [array_sum, array_sum_global,
