@@ -150,6 +150,18 @@ def record_read_second_arr(ary):
     return ary.l[2, 2]
 
 
+def get_charseq(ary, i):
+    return ary[i].n
+
+
+def set_charseq(ary, i, cs):
+    ary[i].n = cs
+
+
+def get_charseq_tuple(ary, i):
+    return ary[i].m, ary[i].n
+
+
 recordtype = np.dtype([('a', np.float64),
                        ('b', np.int32),
                        ('c', np.complex64),
@@ -170,6 +182,9 @@ recordwith2darray = np.dtype([('i', np.int32),
 recordwith2arrays = np.dtype([('k', np.int32, (10, 20)),
                               ('l', np.int32, (6, 12))])
 
+recordwithcharseq = np.dtype([('m', np.int32),
+                              ('n', 'S5')])
+
 class TestRecordDtype(unittest.TestCase):
 
     def _createSampleArrays(self):
@@ -596,5 +611,122 @@ class TestRecordDtypeWithStructArraysAndDispatcher(TestRecordDtypeWithStructArra
     def get_cfunc(self, pyfunc, argspec):
         return _get_cfunc_nopython(pyfunc, argspec)
 
+
+class TestRecordDtypeWithCharSeq(unittest.TestCase):
+    def _createSampleaArray(self):
+        self.refsample1d = np.recarray(3, dtype=recordwithcharseq)
+        self.nbsample1d = np.zeros(3, dtype=recordwithcharseq)
+
+    def _fillData(self, arr):
+        for i in range(arr.size):
+            arr[i]['m'] = i
+            arr[i]['n'] = "%d" % i
+
+        arr[0]['n'] = 'abcde'  # no null-byte
+
+    def setUp(self):
+        self._createSampleaArray()
+        self._fillData(self.refsample1d)
+        self._fillData(self.nbsample1d)
+
+    def get_cfunc(self, pyfunc):
+        rectype = numpy_support.from_dtype(recordwithcharseq)
+        cres = compile_isolated(pyfunc, (rectype[:], types.intp))
+        return cres.entry_point
+
+    def test_return_charseq(self):
+        pyfunc = get_charseq
+        cfunc = self.get_cfunc(pyfunc)
+        for i in range(self.refsample1d.size):
+            expected = pyfunc(self.refsample1d, i)
+            got = cfunc(self.nbsample1d, i)
+            self.assertEqual(expected, got)
+
+    def test_npm_argument_charseq(self):
+        """
+        Test CharSeq as NPM argument
+        """
+
+        def pyfunc(arr, i):
+            return arr[i].n
+
+        identity = jit(lambda x: x)   # an identity function
+
+        @jit(nopython=True)
+        def cfunc(arr, i):
+            return identity(arr[i].n)
+
+        for i in range(self.refsample1d.size):
+            expected = pyfunc(self.refsample1d, i)
+            got = cfunc(self.nbsample1d, i)
+            self.assertEqual(expected, got)
+
+    def test_py_argument_charseq(self):
+        """
+        Test CharSeq as python wrapper argument
+        """
+        pyfunc = set_charseq
+
+        # compile
+        rectype = numpy_support.from_dtype(recordwithcharseq)
+        cres = compile_isolated(pyfunc, (rectype[:], types.intp,
+                                         rectype.typeof('n')))
+        cfunc = cres.entry_point
+
+        for i in range(self.refsample1d.size):
+            chars = "{0}".format(hex(i+10))
+            expected = pyfunc(self.refsample1d, i, chars)
+            got = cfunc(self.nbsample1d, i, chars)
+            np.testing.assert_equal(self.refsample1d, self.nbsample1d)
+
+    def test_py_argument_char_seq_near_overflow(self):
+        """
+        Test strings that are as long as the charseq capacity
+        """
+        pyfunc = set_charseq
+        # compile
+        rectype = numpy_support.from_dtype(recordwithcharseq)
+        cres = compile_isolated(pyfunc, (rectype[:], types.intp,
+                                         rectype.typeof('n')))
+        cfunc = cres.entry_point
+
+        cs_near_overflow = "abcde"
+
+        self.assertEqual(len(cs_near_overflow),
+                         recordwithcharseq['n'].itemsize)
+
+        cfunc(self.nbsample1d, 0, cs_near_overflow)
+        self.assertEqual(self.nbsample1d[0]['n'].decode('ascii'), cs_near_overflow)
+        # Check that we didn't overwrite
+        np.testing.assert_equal(self.refsample1d[1:], self.nbsample1d[1:])
+
+    def test_py_argument_char_seq_truncate(self):
+        """
+        NumPy silently truncates strings to fix inside charseq
+        """
+        pyfunc = set_charseq
+        # compile
+        rectype = numpy_support.from_dtype(recordwithcharseq)
+        cres = compile_isolated(pyfunc, (rectype[:], types.intp,
+                                         rectype.typeof('n')))
+        cfunc = cres.entry_point
+
+        cs_overflowed = "abcdef"
+
+        pyfunc(self.refsample1d, 1, cs_overflowed)
+        cfunc(self.nbsample1d, 1, cs_overflowed)
+        np.testing.assert_equal(self.refsample1d, self.nbsample1d)
+        self.assertEqual(self.refsample1d[1].n,
+                         cs_overflowed[:-1].encode("ascii"))
+
+    def test_return_charseq_tuple(self):
+        pyfunc = get_charseq_tuple
+        cfunc = self.get_cfunc(pyfunc)
+        for i in range(self.refsample1d.size):
+            expected = pyfunc(self.refsample1d, i)
+            got = cfunc(self.nbsample1d, i)
+            self.assertEqual(expected, got)
+
+
 if __name__ == '__main__':
     unittest.main()
