@@ -1,5 +1,7 @@
 from __future__ import print_function, division, absolute_import
 
+import itertools
+
 import numpy
 
 from numba import unittest_support as unittest
@@ -22,6 +24,24 @@ def tuple_second(tup):
 def len_usecase(tup):
     return len(tup)
 
+def eq_usecase(a, b):
+    return a == b
+
+def ne_usecase(a, b):
+    return a != b
+
+def gt_usecase(a, b):
+    return a > b
+
+def ge_usecase(a, b):
+    return a >= b
+
+def lt_usecase(a, b):
+    return a < b
+
+def le_usecase(a, b):
+    return a <= b
+
 
 class TestTupleReturn(TestCase):
 
@@ -90,6 +110,50 @@ class TestOperations(TestCase):
                               [types.UniTuple(types.int64, 3)])
         self.assertPreciseEqual(cr.entry_point((4, 5, 6)), 3)
 
+    def _test_compare(self, pyfunc):
+        def eq(pyfunc, cfunc, args):
+            self.assertIs(cfunc(*args), pyfunc(*args),
+                          "mismatch for arguments %s" % (args,))
+
+        # Same-sized tuples
+        argtypes = [types.Tuple((types.int64, types.float32)),
+                    types.UniTuple(types.int32, 2)]
+        for ta, tb in itertools.product(argtypes, argtypes):
+            cr = compile_isolated(pyfunc, (ta, tb))
+            cfunc = cr.entry_point
+            for args in [((4, 5), (4, 5)),
+                         ((4, 5), (4, 6)),
+                         ((4, 6), (4, 5)),
+                         ((4, 5), (5, 4))]:
+                eq(pyfunc, cfunc, args)
+        # Different-sized tuples
+        argtypes = [types.Tuple((types.int64, types.float32)),
+                    types.UniTuple(types.int32, 3)]
+        cr = compile_isolated(pyfunc, tuple(argtypes))
+        cfunc = cr.entry_point
+        for args in [((4, 5), (4, 5, 6)),
+                     ((4, 5), (4, 4, 6)),
+                     ((4, 5), (4, 6, 7))]:
+            eq(pyfunc, cfunc, args)
+
+    def test_eq(self):
+        self._test_compare(eq_usecase)
+
+    def test_ne(self):
+        self._test_compare(ne_usecase)
+
+    def test_gt(self):
+        self._test_compare(gt_usecase)
+
+    def test_ge(self):
+        self._test_compare(ge_usecase)
+
+    def test_lt(self):
+        self._test_compare(lt_usecase)
+
+    def test_le(self):
+        self._test_compare(le_usecase)
+
 
 if __name__ == '__main__':
     unittest.main()
