@@ -4,10 +4,11 @@ import sys
 
 import numpy
 
-from numba import unittest_support as unittest
+from numba import config, unittest_support as unittest
 from numba.npyufunc.ufuncbuilder import UFuncBuilder, GUFuncBuilder
 from numba import vectorize, guvectorize
-from . import support
+from numba.npyufunc import PyUFunc_One
+from numba.tests import support
 
 
 def add(a, b):
@@ -17,6 +18,10 @@ def add(a, b):
 def equals(a, b):
     return a == b
 
+def mul(a, b):
+    """A multiplication"""
+    return a * b
+
 def guadd(a, b, c):
     """A generalized addition"""
     x, y = c.shape
@@ -92,6 +97,16 @@ class TestUfuncBuilding(unittest.TestCase):
         self.assertTrue(numpy.all(a + a == b))
 
 
+class TestUfuncBuildingJitDisabled(TestUfuncBuilding):
+
+    def setUp(self):
+        self.old_disable_jit = config.DISABLE_JIT
+        config.DISABLE_JIT = False
+
+    def tearDown(self):
+        config.DISABLE_JIT = self.old_disable_jit
+
+
 class TestGUfuncBuilding(unittest.TestCase):
 
     def test_basic_gufunc(self):
@@ -136,6 +151,16 @@ class TestGUfuncBuilding(unittest.TestCase):
         self.assertEqual(b.dtype, numpy.dtype('complex64'))
 
 
+class TestGUfuncBuildingJitDisabled(TestGUfuncBuilding):
+
+    def setUp(self):
+        self.old_disable_jit = config.DISABLE_JIT
+        config.DISABLE_JIT = False
+
+    def tearDown(self):
+        config.DISABLE_JIT = self.old_disable_jit
+
+
 class TestVectorizeDecor(unittest.TestCase):
 
     _supported_identities = [0, 1, None]
@@ -195,6 +220,22 @@ class TestVectorizeDecor(unittest.TestCase):
         with self.assertRaises(ValueError):
             vectorize([sig], identity=2)(add)
 
+    def test_vectorize_no_args(self):
+        a = numpy.linspace(0,1,10)
+        b = numpy.linspace(1,2,10)
+        ufunc = vectorize(add)
+        self.assertTrue(numpy.all(ufunc(a,b) == (a + b)))
+        ufunc2 = vectorize(add)
+        c = numpy.empty(10)
+        ufunc2(a, b, c)
+        self.assertTrue(numpy.all(c == (a + b)))
+
+    def test_vectorize_only_kws(self):
+        a = numpy.linspace(0,1,10)
+        b = numpy.linspace(1,2,10)
+        ufunc = vectorize(identity=PyUFunc_One, nopython=True)(mul)
+        self.assertTrue(numpy.all(ufunc(a,b) == (a * b)))
+
     def test_guvectorize(self):
         ufunc = guvectorize(['(int32[:,:], int32[:,:], int32[:,:])'],
                             "(x,y),(x,y)->(x,y)")(guadd)
@@ -203,6 +244,14 @@ class TestVectorizeDecor(unittest.TestCase):
         self.assertTrue(numpy.all(a + a == b))
         self.assertEqual(b.dtype, numpy.dtype('int32'))
 
+    def test_guvectorize_no_output(self):
+        ufunc = guvectorize(['(int32[:,:], int32[:,:], int32[:,:])'],
+                            "(x,y),(x,y),(x,y)")(guadd)
+        a = numpy.arange(10, dtype='int32').reshape(2, 5)
+        out = numpy.zeros_like(a)
+        ufunc(a, a, out)
+        self.assertTrue(numpy.all(a + a == out))
+
     def test_guvectorize_objectmode(self):
         ufunc = guvectorize(['(int32[:,:], int32[:,:], int32[:,:])'],
                             "(x,y),(x,y)->(x,y)")(guadd_obj)
@@ -232,6 +281,32 @@ class TestVectorizeDecor(unittest.TestCase):
         with self.assertRaises(ValueError):
             guvectorize(*args, identity=2)(add)
 
+    def test_guvectorize_invalid_layout(self):
+        sigs = ['(int32[:,:], int32[:,:], int32[:,:])']
+        # Syntax error
+        with self.assertRaises(ValueError) as raises:
+            guvectorize(sigs, ")-:")(guadd)
+        self.assertIn("bad token in signature", str(raises.exception))
+        # Output shape can't be inferred from inputs
+        with self.assertRaises(NameError) as raises:
+            guvectorize(sigs, "(x,y),(x,y)->(x,z,v)")(guadd)
+        self.assertEqual(str(raises.exception),
+                         "undefined output symbols: v,z")
+        # Arrow but no outputs
+        with self.assertRaises(ValueError) as raises:
+            guvectorize(sigs, "(x,y),(x,y),(x,y)->")(guadd)
+        # (error message depends on Numpy version)
+
+
+class TestVectorizeDecorJitDisabled(TestVectorizeDecor):
+
+    def setUp(self):
+        self.old_disable_jit = config.DISABLE_JIT
+        config.DISABLE_JIT = False
+
+    def tearDown(self):
+        config.DISABLE_JIT = self.old_disable_jit
+
 
 if __name__ == '__main__':
     unittest.main()
