@@ -9,7 +9,7 @@ import warnings
 import numpy as np
 
 import numba.unittest_support as unittest
-from numba import types, typing, utils
+from numba import types, typing, utils, typeof, numpy_support, njit
 from numba.compiler import compile_isolated, Flags, DEFAULT_FLAGS
 from numba.numpy_support import from_dtype
 from numba import vectorize
@@ -17,8 +17,10 @@ from numba.config import PYVERSION
 from numba.typeinfer import TypingError
 from numba.targets import cpu
 from numba.lowering import LoweringError
-from .support import TestCase, CompilationCache, skip_on_numpy_16
+from .support import (TestCase, CompilationCache, skip_on_numpy_16,
+                      is_on_numpy_16)
 
+from numba.typing.npydecl import supported_ufuncs, all_ufuncs
 
 is32bits = tuple.__itemsize__ == 4
 iswindows = sys.platform.startswith('win32')
@@ -28,6 +30,9 @@ enable_pyobj_flags.set("enable_pyobject")
 
 no_pyobj_flags = Flags()
 
+enable_nrt_flags = Flags()
+enable_nrt_flags.set("nrt")
+
 def _unimplemented(func):
     """An 'expectedFailure' like decorator that only expects compilation errors
     caused by unimplemented functions that fail in no-python mode"""
@@ -58,6 +63,14 @@ def _make_unary_ufunc_usecase(ufunc):
     return fn
 
 
+def _make_unary_ufunc_op_usecase(ufunc_op):
+    ldict = {}
+    exec("def fn(x):\n    return {0}(x)".format(ufunc_op), globals(), ldict)
+    fn = ldict["fn"]
+    fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
+    return fn
+
+
 def _make_binary_ufunc_usecase(ufunc):
     ufunc_name = ufunc.__name__
     ldict = {}
@@ -67,6 +80,14 @@ def _make_binary_ufunc_usecase(ufunc):
     return fn
 
 
+def _make_binary_ufunc_op_usecase(ufunc_op):
+    ldict = {}
+    exec("def fn(x,y):\n    return x{0}y".format(ufunc_op), globals(), ldict)
+    fn = ldict["fn"]
+    fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
+    return fn
+
+
 def _as_dtype_value(tyargs, args):
     """Convert python values into numpy scalar objects.
     """
@@ -105,6 +126,29 @@ class TestUFuncs(TestCase):
             ]
         self.cache = CompilationCache()
 
+    def _determine_output_type(self, input_type, int_output_type=None,
+                               float_output_type=None):
+        ty = input_type
+        if isinstance(ty, types.Array):
+            ty = ty.dtype
+
+        if ty in types.signed_domain:
+            if int_output_type:
+                output_type = types.Array(int_output_type, 1, 'C')
+            else:
+                output_type = types.Array(ty, 1, 'C')
+        elif ty in types.unsigned_domain:
+            if int_output_type:
+                output_type = types.Array(int_output_type, 1, 'C')
+            else:
+                output_type = types.Array(ty, 1, 'C')
+        else:
+            if float_output_type:
+                output_type = types.Array(float_output_type, 1, 'C')
+            else:
+                output_type = types.Array(ty, 1, 'C')
+        return output_type
+
     def unary_ufunc_test(self, ufunc, flags=enable_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
                          int_output_type=None, float_output_type=None):
@@ -126,25 +170,8 @@ class TestUFuncs(TestCase):
             if input_type in skip_inputs:
                 continue
 
-            ty = input_type
-            if isinstance(ty, types.Array):
-                ty = ty.dtype
-
-            if ty in types.signed_domain:
-                if int_output_type:
-                    output_type = types.Array(int_output_type, 1, 'C')
-                else:
-                    output_type = types.Array(ty, 1, 'C')
-            elif ty in types.unsigned_domain:
-                if int_output_type:
-                    output_type = types.Array(int_output_type, 1, 'C')
-                else:
-                    output_type = types.Array(ty, 1, 'C')
-            else:
-                if float_output_type:
-                    output_type = types.Array(float_output_type, 1, 'C')
-                else:
-                    output_type = types.Array(ty, 1, 'C')
+            output_type = self._determine_output_type(
+                input_type, int_output_type, float_output_type)
 
             cr = self.cache.compile(pyfunc, (input_type, output_type),
                                     flags=flags)
@@ -199,6 +226,26 @@ class TestUFuncs(TestCase):
                                                expected.dtype, expected)
                         self.fail(msg)
 
+    def unary_op_test(self, operator, flags=enable_nrt_flags,
+                      skip_inputs=[], additional_inputs=[],
+                      int_output_type=None, float_output_type=None):
+        operator_func = _make_unary_ufunc_op_usecase(operator)
+        inputs = list(self.inputs)
+        inputs.extend(additional_inputs)
+        pyfunc = operator_func
+        for input_tuple in inputs:
+            input_operand, input_type = input_tuple
+
+            if ((input_type in skip_inputs) or
+                (not isinstance(input_type, types.Array))):
+                continue
+
+            cr = self.cache.compile(pyfunc, (input_type,),
+                                    flags=flags)
+            cfunc = cr.entry_point
+            expected = pyfunc(input_operand)
+            result = cfunc(input_operand)
+            np.testing.assert_array_almost_equal(expected, result)
 
     def binary_ufunc_test(self, ufunc, flags=enable_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
@@ -216,25 +263,8 @@ class TestUFuncs(TestCase):
             if input_type in skip_inputs:
                 continue
 
-            ty = input_type
-            if isinstance(ty, types.Array):
-                ty = ty.dtype
-
-            if ty in types.signed_domain:
-                if int_output_type:
-                    output_type = types.Array(int_output_type, 1, 'C')
-                else:
-                    output_type = types.Array(ty, 1, 'C')
-            elif ty in types.unsigned_domain:
-                if int_output_type:
-                    output_type = types.Array(int_output_type, 1, 'C')
-                else:
-                    output_type = types.Array(ty, 1, 'C')
-            else:
-                if float_output_type:
-                    output_type = types.Array(float_output_type, 1, 'C')
-                else:
-                    output_type = types.Array(ty, 1, 'C')
+            output_type = self._determine_output_type(
+                input_type, int_output_type, float_output_type)
 
             cr = self.cache.compile(pyfunc, (input_type, input_type, output_type),
                                     flags=flags)
@@ -250,27 +280,56 @@ class TestUFuncs(TestCase):
                 expected = np.zeros(1, dtype=output_type.dtype.name)
             cfunc(input_operand, input_operand, result)
             pyfunc(input_operand, input_operand, expected)
+            np.testing.assert_array_almost_equal(expected, result)
 
-            # Need special checks if NaNs are in results
-            if np.isnan(expected).any() or np.isnan(result).any():
-                self.assertTrue(np.allclose(np.isnan(result), np.isnan(expected)))
-                if not np.isnan(expected).all() and not np.isnan(result).all():
-                    self.assertTrue(np.allclose(result[np.invert(np.isnan(result))],
-                                     expected[np.invert(np.isnan(expected))]))
+    def binary_op_test(self, operator, flags=enable_nrt_flags,
+                       skip_inputs=[], additional_inputs=[],
+                       int_output_type=None, float_output_type=None,
+                       positive_rhs=False):
+        operator_func = _make_binary_ufunc_op_usecase(operator)
+        inputs = list(self.inputs)
+        inputs.extend(additional_inputs)
+        pyfunc = operator_func
+        for input_tuple in inputs:
+            input_operand1, input_type = input_tuple
+            input_dtype = numpy_support.as_dtype(
+                getattr(input_type, "dtype", input_type))
+            input_type1 = input_type
+
+            if input_type in skip_inputs:
+                continue
+
+            if positive_rhs:
+                zero = np.zeros(1, dtype=input_dtype)[0]
+            # If we only use two scalars, the code generator will not
+            # select the ufunctionalized operator, so we mix it up.
+            if isinstance(input_type, types.Array):
+                input_operand0 = input_operand1
+                input_type0 = input_type
+                if positive_rhs and np.any(input_operand1 < zero):
+                    continue
             else:
-                match = np.all(result == expected) or np.allclose(result,
-                                                                  expected)
-                if not match:
-                    msg = '\n'.join(["ufunc '{0}' failed",
-                                     "inputs ({1}):", "{2}",
-                                     "got({3})", "{4}",
-                                     "expected ({5}):", "{6}"
-                                 ]).format(ufunc.__name__,
-                                           input_type, input_operand,
-                                           output_type, result,
-                                           expected.dtype, expected)
-                    self.fail(msg)
+                input_operand0 = (np.random.random(10) * 100).astype(
+                    input_dtype)
+                input_type0 = typeof(input_operand0)
+                if positive_rhs and input_operand1 < zero:
+                    continue
 
+            cr = self.cache.compile(pyfunc, (input_type0, input_type1),
+                                    flags=flags)
+            cfunc = cr.entry_point
+            expected = pyfunc(input_operand0, input_operand1)
+            result = cfunc(input_operand0, input_operand1)
+            np.testing.assert_array_almost_equal(expected, result)
+
+    def binary_int_op_test(self, *args, **kws):
+        if 'skip_inputs' not in kws:
+            kws['skip_inputs'] = []
+        kws['skip_inputs'].extend([
+            types.float32, types.float64,
+            types.Array(types.float32, 1, 'C'),
+            types.Array(types.float64, 1, 'C')])
+        return self.binary_op_test(*args, **kws)
 
     def unary_int_ufunc_test(self, name=None, flags=enable_pyobj_flags):
         self.unary_ufunc_test(name, flags=flags,
@@ -476,7 +535,14 @@ class TestUFuncs(TestCase):
         self.test_square_ufunc(flags=no_pyobj_flags)
 
     def test_reciprocal_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.reciprocal, flags=flags)
+        # reciprocal for integers doesn't make much sense and is problematic
+        # in the case of division by zero, as an inf will overflow float to
+        # int conversions, which is undefined behavior.
+        to_skip = [types.Array(types.uint32, 1, 'C'), types.uint32,
+                   types.Array(types.int32, 1, 'C'), types.int32,
+                   types.Array(types.uint64, 1, 'C'), types.uint64,
+                   types.Array(types.int64, 1, 'C'), types.int64]
+        self.unary_ufunc_test(np.reciprocal, skip_inputs=to_skip, flags=flags)
 
     def test_reciprocal_ufunc_npm(self):
         self.test_reciprocal_ufunc(flags=no_pyobj_flags)
@@ -570,7 +636,20 @@ class TestUFuncs(TestCase):
         self.test_arccosh_ufunc(flags=no_pyobj_flags)
 
     def test_arctanh_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.arctanh, flags=flags)
+        # arctanh is only valid is only finite in the range ]-1, 1[
+        # This means that for any of the integer types it will produce
+        # conversion from infinity/-infinity to integer. That's undefined
+        # behavior in C, so the results may vary from implementation to
+        # implementation. This means that the result from the compiler
+        # used to compile NumPy may differ from the result generated by
+        # llvm. Skipping the integer types in this test avoids failed
+        # tests because of this.
+        to_skip = [types.Array(types.uint32, 1, 'C'), types.uint32,
+                   types.Array(types.int32, 1, 'C'), types.int32,
+                   types.Array(types.uint64, 1, 'C'), types.uint64,
+                   types.Array(types.int64, 1, 'C'), types.int64]
+
+        self.unary_ufunc_test(np.arctanh, skip_inputs=to_skip, flags=flags)
 
     def test_arctanh_ufunc_npm(self):
         self.test_arctanh_ufunc(flags=no_pyobj_flags)
@@ -633,18 +712,16 @@ class TestUFuncs(TestCase):
     def test_bitwise_not_ufunc_npm(self):
         self.test_bitwise_not_ufunc(flags=no_pyobj_flags)
 
-    def test_left_shift_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_int_ufunc_test(np.left_shift, flags=flags)
-
-    def test_left_shift_ufunc_npm(self):
-        self.test_left_shift_ufunc(flags=no_pyobj_flags)
-
-    def test_right_shift_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_int_ufunc_test(np.right_shift, flags=flags)
-
-    def test_right_shift_ufunc_npm(self):
-        self.test_right_shift_ufunc(flags=no_pyobj_flags)
 
+    # Note: there is no entry for left_shift and right_shift as this harness
+    #       is not valid for them. This is so because left_shift and right
+    #       shift implementation in NumPy has undefined behavior (in C-parlance)
+    #       when the second argument is a negative (or bigger than the number
+    #       of bits) value.
+    #       Also, right_shift for negative first arguments also relies on
+    #       implementation defined behavior, although numba warantees "sane"
+    #       behavior (arithmetic shifts on signed integers, logic shifts on
+    #       unsigned integers).
 
     ############################################################################
     # Comparison functions
@@ -778,7 +855,6 @@ class TestUFuncs(TestCase):
     def test_modf_ufunc_npm(self):
         self.test_modf_ufunc(flags=no_pyobj_flags)
 
-
     # Note: there is no entry for ldexp as this harness isn't valid for this
     #       ufunc. this is so because ldexp requires heterogeneous inputs.
     #       However, this ufunc is tested by the TestLoopTypes test classes.
@@ -988,6 +1064,166 @@ class TestUFuncs(TestCase):
             cfunc(x, y, result)
             self.assertTrue(np.all(result == expected))
 
+    def test_implicit_output_npm(self):
+        with self.assertRaises(TypeError):
+            def myadd(a0, a1):
+                return np.add(a0, a1)
+            arr_ty = types.Array(types.uint64, 1, 'C')
+            cr = compile_isolated(myadd, (arr_ty, arr_ty),
+                                  flags=no_pyobj_flags)
+
+    def test_broadcast_implicit_output_npm_nrt(self):
+        def pyfunc(a0, a1):
+            return np.add(a0, a1)
+
+        input1_operands = [
+            np.arange(3, dtype='u8'),
+            np.arange(3*3, dtype='u8').reshape(3,3),
+            np.arange(3*3*3, dtype='u8').reshape(3,3,3),
+            np.arange(3, dtype='u8').reshape(3,1),
+            np.arange(3, dtype='u8').reshape(1,3),
+            np.arange(3, dtype='u8').reshape(3,1,1),
+            np.arange(3*3, dtype='u8').reshape(3,3,1),
+            np.arange(3*3, dtype='u8').reshape(3,1,3),
+            np.arange(3*3, dtype='u8').reshape(1,3,3)]
+
+        input2_operands = input1_operands
+
+        for x, y in itertools.product(input1_operands, input2_operands):
+
+            input1_type = types.Array(types.uint64, x.ndim, 'C')
+            input2_type = types.Array(types.uint64, y.ndim, 'C')
+
+            cr = self.cache.compile(pyfunc, (input1_type, input2_type),
+                                    flags=enable_nrt_flags)
+            cfunc = cr.entry_point
+
+            expected = np.add(x, y)
+            result = cfunc(x, y)
+            np.testing.assert_array_equal(expected, result)
+
+    def test_implicit_output_layout(self):
+        def pyfunc(a0, a1):
+            return np.add(a0, a1)
+        X = np.linspace(0, 1, 20).reshape(4, 5)
+        Y = np.array(X, order='F')
+        Xty = typeof(X)
+        assert X.flags.c_contiguous and Xty.layout == 'C'
+        Yty = typeof(Y)
+        assert Y.flags.f_contiguous and Yty.layout == 'F'
+
+        cr0 = self.cache.compile(pyfunc, (Xty, Yty), flags=enable_nrt_flags)
+        expected0 = np.add(X, Y)
+        result0 = cr0.entry_point(X, Y)
+        self.assertEqual(expected0.flags.c_contiguous,
+                         result0.flags.c_contiguous)
+        self.assertEqual(expected0.flags.f_contiguous,
+                         result0.flags.f_contiguous)
+        np.testing.assert_array_equal(expected0, result0)
+
+        cr1 = self.cache.compile(pyfunc, (Yty, Yty), flags=enable_nrt_flags)
+        expected1 = np.add(Y, Y)
+        result1 = cr1.entry_point(Y, Y)
+        self.assertEqual(expected1.flags.c_contiguous,
+                         result1.flags.c_contiguous)
+        self.assertEqual(expected1.flags.f_contiguous,
+                         result1.flags.f_contiguous)
+        np.testing.assert_array_equal(expected1, result1)
+
+    # ____________________________________________________________
+    # Array operators
+
+    def test_unary_positive_array_op(self):
+        self.unary_op_test('+')
+
+    def test_unary_negative_array_op(self):
+        self.unary_op_test('-')
+
+    def test_unary_invert_array_op(self):
+        self.unary_op_test('~', skip_inputs=[
+            types.float32, types.float64,
+            types.Array(types.float32, 1, 'C'),
+            types.Array(types.float64, 1, 'C')])
+
+    def test_add_array_op(self):
+        self.binary_op_test('+')
+
+    def test_subtract_array_op(self):
+        self.binary_op_test('-')
+
+    def test_multiply_array_op(self):
+        self.binary_op_test('*')
+
+    def test_divide_array_op(self):
+        int_out_type = None
+        if PYVERSION >= (3, 0):
+            int_out_type = types.float64
+        self.binary_op_test('/', int_output_type=int_out_type)
+
+    def test_floor_divide_array_op(self):
+        self.binary_op_test('//')
+
+    def test_remainder_array_op(self):
+        self.binary_op_test('%')
+
+    def test_power_array_op(self):
+        self.binary_op_test('**')
+
+    def test_left_shift_array_op(self):
+        self.binary_int_op_test('<<', positive_rhs=True)
+
+    def test_right_shift_array_op(self):
+        self.binary_int_op_test('>>', positive_rhs=True)
+
+    def test_bitwise_and_array_op(self):
+        self.binary_int_op_test('&')
+
+    def test_bitwise_or_array_op(self):
+        self.binary_int_op_test('|')
+
+    def test_bitwise_xor_array_op(self):
+        self.binary_int_op_test('^')
+
+    def test_equal_array_op(self):
+        self.binary_op_test('==')
+
+    def test_greater_array_op(self):
+        self.binary_op_test('>')
+
+    def test_greater_equal_array_op(self):
+        self.binary_op_test('>=')
+
+    def test_less_array_op(self):
+        self.binary_op_test('<')
+
+    def test_less_equal_array_op(self):
+        self.binary_op_test('<=')
+
+    def test_not_equal_array_op(self):
+        self.binary_op_test('!=')
+
+    def test_unary_positive_array_op(self):
+        '''
+        Verify that the unary positive operator copies values, and doesn't
+        just alias to the input array (mirrors normal Numpy/Python
+        interaction behavior).
+        '''
+        # Test originally from @gmarkall
+        def f(a1):
+            a2 = +a1
+            a1[0] = 3
+            a2[1] = 4
+            return a2
+
+        a1 = np.zeros(10)
+        a2 = f(a1)
+        self.assertTrue(a1[0] != a2[0] and a1[1] != a2[1])
+        a3 = np.zeros(10)
+        a4 = njit(f)(a3)
+        self.assertTrue(a3[0] != a4[0] and a3[1] != a4[1])
+        np.testing.assert_array_equal(a1, a3)
+        np.testing.assert_array_equal(a2, a4)
+
 
 class TestScalarUFuncs(TestCase):
     """check the machinery of ufuncs works when the result is an scalar.
@@ -1106,8 +1342,13 @@ class TestUfuncIssues(TestCase):
         self.assertEqual(foo(1j, 1j), cr.entry_point(1j, 1j))
 
 
-class TestLoopTypes(TestCase):
+class _TestLoopTypes(TestCase):
     """Test code generation for the different loop types defined by ufunc.
+
+    This class tests the ufuncs without forcing no-python mode. Subclasses
+    of this class tweak it so they tests no-python mode support for the
+    different ufuncs.
+
     This test relies on class variables to configure the test. Subclasses
     of this class can just override some of these variables to check other
     ufuncs in a different compilation context. The variables supported are:
@@ -1128,30 +1369,22 @@ class TestLoopTypes(TestCase):
     'types'.
     """
 
-    _ufuncs = [np.add, np.subtract, np.multiply, np.divide, np.logaddexp,
-               np.logaddexp2, np.true_divide, np.floor_divide, np.negative,
-               np.power, np.remainder, np.mod, np.fmod, np.abs, np.absolute,
-               np.rint, np.sign, np.conj, np.exp, np.exp2, np.log, np.log2,
-               np.log10, np.expm1, np.log1p, np.sqrt, np.square, np.reciprocal,
-               np.conjugate, np.sin, np.cos, np.tan, np.arcsin, np.arccos,
-               np.arctan, np.arctan2, np.hypot, np.sinh, np.cosh, np.tanh,
-               np.arcsinh, np.arccosh, np.arctanh, np.deg2rad, np.rad2deg,
-               np.degrees, np.radians, np.bitwise_and, np.bitwise_or,
-               np.bitwise_xor, np.bitwise_not, np.invert, np.left_shift,
-               np.right_shift, np.greater, np.greater_equal, np.less,
-               np.less_equal, np.not_equal, np.equal, np.logical_and,
-               np.logical_or, np.logical_xor, np.logical_not, np.maximum,
-               np.minimum, np.fmax, np.fmin, np.isfinite, np.isinf, np.isnan,
-               np.signbit, np.copysign, np.nextafter, np.modf, np.ldexp,
-               np.frexp, np.floor, np.ceil, np.trunc, np.spacing ]
+    _ufuncs = all_ufuncs[:]
+    _ufuncs.remove(np.left_shift) # has its own test class
     _compile_flags = enable_pyobj_flags
-    _skip_types='O'
+    _skip_types = 'OegG'
 
-    def _arg_for_type(self, a_letter_type):
+    # Skip datetime64 'M' and timedelta64 'm' on numpy 1.6
+    if is_on_numpy_16:
+        _skip_types += 'Mm'
+
+    def _arg_for_type(self, a_letter_type, index=0):
         """return a suitable array argument for testing the letter type"""
-        if a_letter_type in 'bBhHiIlLqQ':
+        if a_letter_type in 'bhilq':
             # an integral
-            return np.array([2, 3, 4, 0], dtype=a_letter_type)
+            return np.array([1, 4, 0, -2], dtype=a_letter_type)
+        if a_letter_type in 'BHILQ':
+            return np.array([1, 2, 4, 0], dtype=a_letter_type)
         elif a_letter_type in '?':
             # a boolean
             return np.array([True, False, False, True], dtype=a_letter_type)
@@ -1170,7 +1403,8 @@ class TestLoopTypes(TestCase):
             return np.array([1.5, -3.5, 0.0, float('nan')], dtype=a_letter_type)
         elif a_letter_type in 'FD':
             # complex
-            return np.array([-1.0j, 1.5 + 1.5j, 1j * float('nan'), 0j], dtype=a_letter_type)
+            return np.array([-1.0j, 1.5 + 1.5j, 1j * float('nan'), 0j],
+                            dtype=a_letter_type)
         else:
             raise RuntimeError("type %r not understood" % (a_letter_type,))
 
@@ -1182,11 +1416,12 @@ class TestLoopTypes(TestCase):
         # fail in no python mode. Usually the last loop in ufuncs is an all
         # object fallback
         supported_types = getattr(self, '_supported_types', [])
-        skip_types = getattr(self, 'skip_types', [])
-        if any(l not in supported_types or l in skip_types
-               for l in letter_types):
+        if (supported_types and
+            any(l not in supported_types for l in letter_types)):
+            return
+        skip_types = getattr(self, '_skip_types', [])
+        if any(l in skip_types for l in letter_types):
             return
-
         # if the test case requires some types to be present, skip loops
         # not involving any of those types.
         required_types = getattr(self, '_required_types', [])
@@ -1202,7 +1437,8 @@ class TestLoopTypes(TestCase):
         cr = compile_isolated(fn, arg_nbty, flags=self._compile_flags)
 
         # Ensure a good mix of input values
-        c_args = [self._arg_for_type(t).repeat(2) for t in dtypes]
+        c_args = [self._arg_for_type(t, index=index).repeat(2)
+                  for index, t in enumerate(dtypes)]
         for arr in c_args:
             self.random.shuffle(arr)
         py_args = [a.copy() for a in c_args]
@@ -1217,124 +1453,183 @@ class TestLoopTypes(TestCase):
             prec = 'single' if c_arg.dtype.char in 'fF' else 'exact'
             prec = 'double' if c_arg.dtype.char in 'dD' else prec
             for c, py in zip(c_arg, py_arg):
-                self.assertPreciseEqual(py, c, prec=prec,
-                    msg="ufunc %s; arrays differ (%s):\n args:%s expected %r, got %r" % (ufunc.__name__, c_args, prec, py_arg, c_arg))
+                msg = '\n'.join(["ufunc '{0}' arrays differ ({1}):",
+                                 "args: {2}", "expected {3}", "got {4}"])
+                msg = msg.format(ufunc.__name__, c_args, prec, py_arg, c_arg)
+                try:
+                    self.assertPreciseEqual(py, c, prec=prec, msg=msg)
+                except AssertionError:
+                    if not self._is_acceptable_error(py, c):
+                        raise
 
-    def _check_ufunc_loops(self, ufunc):
-        fn = _make_ufunc_usecase(ufunc)
-        _failed_loops = []
+    def _is_acceptable_error(self, py, c):
+        """
+        Result mismatch due to differing error handling.
+        """
+        def get_leftmost_bit(dtype):
+            return dtype.type(1 << (dtype.itemsize * 8 - 1))
+
+        if np.isnan(c) or np.isnan(py):
+            if 0 in [c, py]:
+                # If one is NaN and the other is zero
+                msg = "mismatch for nan values: expected {0}; got {1}"
+                print(msg.format(py, c))
+                # This is not occurring any more
+
+        elif np.abs(c) == 0 and np.abs(py) == np.abs(c):
+            # If both are zeros and they are differ by sign
+            msg = "mismatch sign for 0: expected {0}; got {1}"
+            print(msg.format(py, c))
+            return True
+
+        elif ((c == 0 and py == get_leftmost_bit(py.dtype)) or
+                  (py == 0 and c == get_leftmost_bit(py.dtype))):
+            msg = "mismatch left-most bit: expected {0}; got {1}"
+            print(msg.format(py, c))
+            return True
+
+        elif isinstance(c, complex) and isinstance(py, complex):
+            # Recurse into real and imag of a complex
+            return (self._is_acceptable_error(c.real, py.real)
+                    and self._is_acceptable_error(c.imag, py.imag))
+
+        # Match?
+        return py == c
+
+    @classmethod
+    def _check_ufunc_loops(cls, ufunc):
         for loop in ufunc.types:
-            try:
-                self._check_loop(fn, ufunc, loop)
-            except AssertionError as e:
-                import traceback
-                traceback.print_exc()
-                _failed_loops.append('{2} {0}:{1}'.format(loop, str(e),
-                                                          ufunc.__name__))
+            cls._inject_test(ufunc, loop)
 
-        return _failed_loops
+    @classmethod
+    def _inject_test(cls, ufunc, loop):
+        def test_template(self):
+            fn = _make_ufunc_usecase(ufunc)
+            self._check_loop(fn, ufunc, loop)
+        setattr(cls, "test_{0}_{1}".format(ufunc.__name__,
+                                           loop.replace('->', '_')),
+                test_template)
 
-    def test_ufunc_loops(self):
-        failed_ufuncs = []
-        failed_loops_count = 0
-        for ufunc in self._ufuncs:
-            failed_loops = self._check_ufunc_loops(ufunc)
-            if failed_loops:
-                failed_loops_count += len(failed_loops)
-                msg = 'ufunc {0} failed in loops:\n\t{1}\n\t'.format(
-                    ufunc.__name__,
-                    '\n\t'.join(failed_loops))
-                failed_ufuncs.append(msg)
-
-        if failed_ufuncs:
-            msg = 'Failed {0} ufuncs, {1} loops:\n{2}'.format(
-                len(failed_ufuncs), failed_loops_count,
-                '\n'.join(failed_ufuncs))
-
-            self.fail(msg=msg)
+    @classmethod
+    def autogenerate(cls):
+        for ufunc in cls._ufuncs:
+            cls._check_ufunc_loops(ufunc)
 
 
+class TestLoopTypes(_TestLoopTypes):
+    pass
 
-class TestLoopTypesFloatNoPython(TestLoopTypes):
+TestLoopTypes.autogenerate()
+
+
+class TestLoopTypesIntNoPython(_TestLoopTypes):
     _compile_flags = no_pyobj_flags
+    _ufuncs = supported_ufuncs[:]
+    # reciprocal needs a special test due to issue #757
+    _ufuncs.remove(np.reciprocal)
+    _ufuncs.remove(np.left_shift) # has its own test class
+    _ufuncs.remove(np.right_shift) # has its own test class
+    _required_types = '?bBhHiIlLqQ'
+    _skip_types = 'fdFDmMO' + _TestLoopTypes._skip_types
 
-    _ufuncs = [np.add, np.subtract, np.multiply, np.divide, np.logaddexp,
-               np.logaddexp2, np.true_divide, np.floor_divide, np.negative,
-               np.remainder, np.mod, np.power, np.abs, np.absolute,
-               np.sign, np.conj, np.exp, np.exp2, np.log, np.log2,
-               np.log10, np.expm1, np.log1p, np.sqrt, np.square, np.reciprocal,
-               np.conjugate, np.sin, np.cos, np.tan, np.arcsin, np.arccos,
-               np.arctan, np.arctan2, np.sinh, np.cosh, np.tanh,
-               np.arcsinh, np.arccosh, np.arctanh, np.deg2rad, np.rad2deg,
-               np.degrees, np.radians,
-               np.floor, np.ceil, np.trunc,
-               np.greater, np.greater_equal, np.less, np.less_equal,
-               np.not_equal, np.equal, np.logical_and, np.logical_or,
-               np.logical_xor, np.logical_not, np.maximum, np.minimum,
-               np.fmax, np.fmin, np.isnan, np.bitwise_and, np.bitwise_or,
-               np.bitwise_xor, np.bitwise_not, np.invert, np.left_shift,
-               np.right_shift, np.isinf, np.isfinite, np.copysign,
-               np.nextafter, np.spacing, np.ldexp, np.fmod ]
+TestLoopTypesIntNoPython.autogenerate()
 
-    _supported_types = '?fd'
-
-if not iswindows:  # NaN handling is inconsistent!
-    TestLoopTypesFloatNoPython._ufuncs.append(np.signbit)
-
-class TestLoopTypesIntNoPython(TestLoopTypes):
+class TestLoopTypesIntReciprocalNoPython(_TestLoopTypes):
     _compile_flags = no_pyobj_flags
+    _ufuncs = [np.reciprocal] # issue #757
+    _required_types = 'bBhHiIlLqQ'
+    _skip_types = 'fdFDmMO' + _TestLoopTypes._skip_types
 
-    _ufuncs = [np.add, np.subtract, np.multiply, np.divide, np.logaddexp,
-               np.logaddexp2, np.true_divide, np.floor_divide, np.negative,
-               np.remainder, np.mod, np.power, np.abs, np.absolute,
-               np.sign, np.conj, np.exp, np.exp2, np.log, np.log2,
-               np.log10, np.expm1, np.log1p, np.sqrt, np.square,  ## np.reciprocal,
-               np.conjugate, np.sin, np.cos, np.tan, np.arcsin, np.arccos,
-               np.arctan, np.arctan2, np.sinh, np.cosh, np.tanh,
-               np.arcsinh, np.arccosh, np.arctanh, np.deg2rad, np.rad2deg,
-               np.degrees, np.radians,
-               np.floor, np.ceil, np.trunc,
-               np.greater, np.greater_equal, np.less, np.less_equal,
-               np.not_equal, np.equal, np.logical_and, np.logical_or,
-               np.logical_xor, np.logical_not, np.maximum, np.minimum,
-               np.fmax, np.fmin, np.isnan, np.bitwise_and, np.bitwise_or,
-               np.bitwise_xor, np.bitwise_not, np.invert, np.left_shift,
-               np.right_shift, np.isinf, np.isfinite, np.signbit, np.copysign,
-               np.nextafter, np.spacing, np.ldexp, np.fmod ]
+    def _arg_for_type(self, a_letter_type, index=0):
+        res = super(self.__class__, self)._arg_for_type(a_letter_type,
+                                                        index=index)
+        # avoid 0 as argument, as it triggers undefined behavior that my differ
+        # in results from numba to the compiler used to compile NumPy
+        res[res == 0] = 42
+        return res
 
-    _supported_types = '?bBhHiIlLqQ'
+TestLoopTypesIntReciprocalNoPython.autogenerate()
 
-
-class TestLoopTypesComplexNoPython(TestLoopTypes):
+class TestLoopTypesIntLeftShiftNoPython(_TestLoopTypes):
     _compile_flags = no_pyobj_flags
-    _ufuncs = [np.negative, np.add, np.subtract, np.multiply, np.divide,
-               np.true_divide, np.floor_divide, np.power, np.sign, np.abs,
-               np.absolute, np.rint, np.conj, np.conjugate, np.exp, np.exp2,
-               np.log, np.log2, np.log10, np.expm1, np.log1p, np.sqrt,
-               np.square, np.reciprocal, np.sin, np.cos, np.tan, np.arcsin,
-               np.arccos, np.arctan, np.sinh, np.cosh, np.tanh, np.arcsinh,
-               np.arccosh, np.arctanh, np.greater, np.greater_equal, np.less,
-               np.less_equal, np.not_equal, np.equal, np.logical_and,
-               np.logical_or, np.logical_xor, np.logical_not, np.maximum,
-               np.minimum, np.fmax, np.fmin, np.isnan, np.isinf, np.isfinite ]
+    _ufuncs = [np.left_shift]
+    _required_types = 'bBhHiIlLqQ'
+    _skip_types = 'fdFDmMO' + _TestLoopTypes._skip_types
+
+    def _arg_for_type(self, a_letter_type, index=0):
+        res = super(self.__class__, self)._arg_for_type(a_letter_type,
+                                                        index=index)
+        # Shifting by a negative amount (argument with index 1) is undefined
+        # behavior in C. It is also undefined behavior in numba. In the same
+        # sense, it is also undefined behavior when the shift amount is larger
+        # than the number of bits in the shifted integer.
+        # To avoid problems in the test, the values are clamped (clipped) so
+        # that 0 <= shift_amount < bitcount(shifted_integer)
+        if index == 1:
+            bit_count = res.dtype.itemsize * 8
+            res = np.clip(res, 0, bit_count-1)
+        return res
+
+TestLoopTypesIntLeftShiftNoPython.autogenerate()
+
+class TestLoopTypesIntRightShiftNoPython(_TestLoopTypes):
+    _compile_flags = no_pyobj_flags
+    _ufuncs = [np.right_shift]
+    _required_types = 'bBhHiIlLqQ'
+    _skip_types = 'fdFDmMO' + _TestLoopTypes._skip_types
+
+    def _arg_for_type(self, a_letter_type, index=0):
+        res = super(self.__class__, self)._arg_for_type(a_letter_type,
+                                                        index=index)
+        # Shifting by a negative amount (argument with index 1) is undefined
+        # behavior in C. It is also undefined behavior in numba. In the same
+        # sense, it is also undefined behavior when the shift amount is larger
+        # than the number of bits in the shifted integer.
+        # To avoid problems in the test, the values are clamped (clipped) so
+        # that 0 <= shift_amount < bitcount(shifted_integer)
+        if index == 1:
+            bit_count = res.dtype.itemsize * 8
+            res = np.clip(res, 0, bit_count-1)
+
+        # Right shift has "implementation defined behavior" when the number
+        # shifted is negative (in C). In numba, right shift for signed integers
+        # is "arithmetic" while for unsigned integers is "logical".
+        # This test compares against the NumPy implementation, that relies
+        # on "implementation defined behavior", so the test could be a false
+        # failure if the compiler used to compile NumPy doesn't follow the same
+        # policy.
+        # Hint: do not rely on right shifting negative numbers in NumPy.
+        if index == 0:
+            res = np.abs(res)
+        return res
+
+TestLoopTypesIntRightShiftNoPython.autogenerate()
+
+class TestLoopTypesFloatNoPython(_TestLoopTypes):
+    _compile_flags = no_pyobj_flags
+    _ufuncs = supported_ufuncs[:]
+    if iswindows:
+        _ufuncs.remove(np.signbit) # TODO: fix issue #758
+    _required_types = 'fd'
+    _skip_types = 'FDmMO' + _TestLoopTypes._skip_types
+
+TestLoopTypesFloatNoPython.autogenerate()
+
+class TestLoopTypesComplexNoPython(_TestLoopTypes):
+    _compile_flags = no_pyobj_flags
+    _ufuncs = supported_ufuncs[:]
 
     # Test complex types
-    # note that some loops like "abs" contain reals as results, hence the
-    # _supported_types. Boolean functions will return '?', so it is also
-    # included
-    _supported_types = 'FDfd?'
+    # Every loop containing a complex argument must be tested
     _required_types = 'FD'
+    _skip_types = 'mMO' + _TestLoopTypes._skip_types
 
+TestLoopTypesComplexNoPython.autogenerate()
 
 @skip_on_numpy_16
-class TestLoopTypesDatetimeNoPython(TestLoopTypes):
+class TestLoopTypesDatetimeNoPython(_TestLoopTypes):
     _compile_flags = no_pyobj_flags
-    _ufuncs = [np.absolute, np.negative, np.sign,
-               np.add, np.subtract, np.multiply,
-               np.divide, np.true_divide, np.floor_divide,
-               np.equal, np.not_equal, np.less, np.less_equal,
-               np.greater, np.greater_equal, np.maximum,
-               np.minimum, np.fmax, np.fmin ]
+    _ufuncs = supported_ufuncs[:]
 
     # NOTE: the full list of ufuncs supporting datetime64 and timedelta64
     # types in Numpy is:
@@ -1344,7 +1639,6 @@ class TestLoopTypesDatetimeNoPython(TestLoopTypes):
     #  'true_divide']
 
     # Test datetime64 and timedelta64 types.
-    _supported_types = 'mMqd?'
     _required_types = 'mM'
 
     # Test various units combinations (TestLoopTypes is only able to test
@@ -1427,6 +1721,7 @@ class TestLoopTypesDatetimeNoPython(TestLoopTypes):
                       np.greater, np.greater_equal]:
             self._check_comparison(ufunc)
 
+TestLoopTypesDatetimeNoPython.autogenerate()
 
 class TestUFuncBadArgsNoPython(TestCase):
     _compile_flags = no_pyobj_flags
