@@ -155,11 +155,19 @@ class OpaqueType(Type):
 
 
 class Boolean(Type):
-    pass
+
+    def cast_python_value(self, value):
+        return bool(value)
+
+
+class Number(Type):
+    """
+    Base class for number types.
+    """
 
 
 @utils.total_ordering
-class Integer(Type):
+class Integer(Number):
     def __init__(self, *args, **kws):
         super(Integer, self).__init__(*args, **kws)
         # Determine bitwidth
@@ -181,7 +189,7 @@ class Integer(Type):
 
 
 @utils.total_ordering
-class Float(Type):
+class Float(Number):
     def __init__(self, *args, **kws):
         super(Float, self).__init__(*args, **kws)
         # Determine bitwidth
@@ -199,7 +207,7 @@ class Float(Type):
 
 
 @utils.total_ordering
-class Complex(Type):
+class Complex(Number):
     def __init__(self, name, underlying_float, **kwargs):
         super(Complex, self).__init__(name, **kwargs)
         self.underlying_float = underlying_float
@@ -303,7 +311,7 @@ class VarArg(Type):
         return self.dtype
 
 
-class Module(Type):
+class Module(Dummy):
     def __init__(self, pymod):
         self.pymod = pymod
         super(Module, self).__init__("Module(%s)" % pymod)
@@ -324,19 +332,77 @@ class Macro(Type):
         return self.template
 
 
-class Function(Type):
+class Callable(Type):
+    """
+    Base class for callables.
+    """
+
+    def get_call_type(self, context, args, kws):
+        """
+        Using the typing *context*, resolve the callable's signature for
+        the given arguments.  A signature object is returned, or None.
+        """
+        raise NotImplementedError
+
+
+class Function(Callable, Opaque):
     def __init__(self, template):
         self.template = template
         name = "%s(%s)" % (self.__class__.__name__, template.key)
-        # TODO template is mutable.  Should use different naming scheme
         super(Function, self).__init__(name)
 
     @property
     def key(self):
         return self.template
 
-    def extend(self, template):
-        self.template.cases.extend(template.cases)
+    def get_call_type(self, context, args, kws):
+        return self.template(context).apply(args, kws)
+
+
+class DTypeSpec(Opaque):
+    """
+    Base class for types usable as "dtype" arguments to various Numpy APIs
+    (e.g. np.empty()).
+    """
+
+    @property
+    def dtype(self):
+        raise NotImplementedError
+
+
+class NumberClass(Callable, DTypeSpec):
+    """
+    Type class for number classes (e.g. "np.float64").
+    """
+
+    def __init__(self, instance_type, template):
+        self.instance_type = instance_type
+        self.template = template
+        name = "type(%s)" % (instance_type,)
+        super(NumberClass, self).__init__(name)
+
+    def get_call_type(self, context, args, kws):
+        return self.template(context).apply(args, kws)
+
+    @property
+    def dtype(self):
+        return self.instance_type
+
+
+class DType(DTypeSpec):
+    """
+    Type class for Numpy dtypes.
+    """
+
+    def __init__(self, dtype):
+        assert isinstance(dtype, Type)
+        self._dtype = dtype
+        name = "dtype(%s)" % (dtype,)
+        super(DTypeSpec, self).__init__(name)
+
+    @property
+    def dtype(self):
+        return self._dtype
 
 
 class WeakType(Type):
@@ -367,12 +433,18 @@ class WeakType(Type):
         return Type.__hash__(self)
 
 
-class Dispatcher(WeakType):
+class Dispatcher(WeakType, Callable, Dummy):
 
     def __init__(self, overloaded):
         self._store_object(overloaded)
         super(Dispatcher, self).__init__("Dispatcher(%s)" % overloaded)
 
+    def get_call_type(self, context, args, kws):
+        template, args, kws = self.overloaded.get_call_template(args, kws)
+        sig = template(context).apply(args, kws)
+        sig.pysig = self.pysig
+        return sig
+
     @property
     def overloaded(self):
         """
@@ -394,13 +466,32 @@ class ExternalFunctionPointer(Function):
     *get_pointer* is a Python function taking an object
     and returning the raw pointer value as an int.
     """
-
     def __init__(self, sig, get_pointer, cconv=None):
-        from . import typing
+        from .typing.templates import (AbstractTemplate, make_concrete_template,
+                                       signature)
+        if sig.return_type == ffi_forced_object:
+            raise TypeError("Cannot return a pyobject from a external function")
         self.sig = sig
+        self.requires_gil = any(a == ffi_forced_object for a in self.sig.args)
         self.get_pointer = get_pointer
         self.cconv = cconv
-        template = typing.make_concrete_template("CFuncPtr", sig, [sig])
+        if self.requires_gil:
+            class GilRequiringDefn(AbstractTemplate):
+                key = self.sig
+
+                def generic(self, args, kws):
+                    if kws:
+                        raise TypeError("does not support keyword arguments")
+                    # Make ffi_forced_object a bottom type to allow any type to be
+                    # casted to it. This is the only place that support
+                    # ffi_forced_object.
+                    coerced = [actual if formal == ffi_forced_object else formal
+                               for actual, formal
+                               in zip(args, self.key.args)]
+                    return signature(self.key.return_type, *coerced)
+            template = GilRequiringDefn
+        else:
+            template = make_concrete_template("CFuncPtr", sig, [sig])
         super(ExternalFunctionPointer, self).__init__(template)
 
     @property
@@ -803,6 +894,22 @@ class Array(Buffer):
         return self.dtype, self.ndim, self.layout, self.mutable
 
 
+class ArrayCTypes(Type):
+    """
+    This is the type for `numpy.ndarray.ctypes`.
+    """
+    def __init__(self, arytype):
+        # This depends on the ndim for the shape and strides attributes,
+        # even though they are not implemented, yet.
+        self.ndim = arytype.ndim
+        name = "ArrayCType(ndim={0})".format(self.ndim)
+        super(ArrayCTypes, self).__init__(name, param=True)
+
+    @property
+    def key(self):
+        return self.ndim
+
+
 class NestedArray(Array):
     """
     A NestedArray is an array nested within a structured type (which are "void"
@@ -846,7 +953,14 @@ class NestedArray(Array):
     def key(self):
         return self.dtype, self.shape
 
-class UniTuple(IterableType):
+
+class BaseTuple(Type):
+    """
+    The base class for all tuple types (with a known size).
+    """
+
+
+class UniTuple(IterableType, BaseTuple):
 
     def __init__(self, dtype, count):
         self.dtype = dtype
@@ -902,7 +1016,8 @@ class UniTupleIter(IteratorType):
         return self.unituple
 
 
-class Tuple(Type):
+class Tuple(BaseTuple):
+
     def __init__(self, types):
         self.types = tuple(types)
         self.count = len(self.types)
@@ -967,6 +1082,23 @@ class EphemeralPointer(CPointer):
     """
 
 
+class EphemeralArray(Type):
+    """
+    Similar to EphemeralPointer, but pointing to an array of elements,
+    rather than a single one.  The array size must be known at compile-time.
+    """
+
+    def __init__(self, dtype, count):
+        self.dtype = dtype
+        self.count = count
+        name = "*%s[%d]" % (dtype, count)
+        super(EphemeralArray, self).__init__(name, param=True)
+
+    @property
+    def key(self):
+        return self.dtype, self.count
+
+
 class Object(Type):
     mutable = True
 
@@ -1024,7 +1156,7 @@ class NoneType(Opaque):
         return Optional(other)
 
 
-class ExceptionType(Phantom):
+class ExceptionType(Callable, Phantom):
     """
     The type of exception classes (not instances).
     """
@@ -1035,6 +1167,11 @@ class ExceptionType(Phantom):
         self.exc_class = exc_class
         super(ExceptionType, self).__init__(name, param=True)
 
+    def get_call_type(self, context, args, kws):
+        from . import typing
+        return_type = ExceptionInstance(self.exc_class)
+        return typing.signature(return_type)
+
     @property
     def key(self):
         return self.exc_class
@@ -1061,6 +1198,8 @@ class Slice3Type(Type):
     pass
 
 
+
+
 # Utils
 
 def is_int_tuple(x):
@@ -1076,6 +1215,7 @@ def is_int_tuple(x):
 
 
 pyobject = Opaque('pyobject')
+ffi_forced_object = Opaque('ffi_forced_object')
 none = NoneType('none')
 Any = Phantom('any')
 string = Dummy('str')
@@ -1084,6 +1224,9 @@ string = Dummy('str')
 # Can only pass it around
 voidptr = Opaque('void*')
 
+# For NRT GC
+meminfo_pointer = Opaque("MemInfo*")
+
 boolean = bool_ = Boolean('bool')
 
 byte = uint8 = Integer('uint8')
@@ -1117,8 +1260,11 @@ sign_type = Phantom('sign')
 
 range_iter32_type = RangeIteratorType('range_iter32', int32)
 range_iter64_type = RangeIteratorType('range_iter64', int64)
+unsigned_range_iter64_type = RangeIteratorType('unsigned_range_iter64', uint64)
 range_state32_type = RangeType('range_state32', range_iter32_type)
 range_state64_type = RangeType('range_state64', range_iter64_type)
+unsigned_range_state64_type = RangeType('unsigned_range_state64',
+                                        unsigned_range_iter64_type)
 
 # slice2_type = Type('slice2_type')
 slice3_type = Slice3Type('slice3_type')
@@ -1248,4 +1394,5 @@ f8
 c8
 c16
 optional
+ffi_forced_object
 '''.split()
