@@ -77,6 +77,10 @@ class Range(ConcreteTemplate):
         signature(types.range_state64_type, types.int64, types.int64),
         signature(types.range_state64_type, types.int64, types.int64,
                   types.int64),
+        signature(types.unsigned_range_state64_type, types.uint64),
+        signature(types.unsigned_range_state64_type, types.uint64, types.uint64),
+        signature(types.unsigned_range_state64_type, types.uint64, types.uint64,
+                  types.uint64),
     ]
 
 
@@ -308,32 +312,65 @@ class UnorderedCmpOp(ConcreteTemplate):
 class CmpOpLt(OrderedCmpOp):
     key = '<'
 
-
 @builtin
 class CmpOpLe(OrderedCmpOp):
     key = '<='
 
-
 @builtin
 class CmpOpGt(OrderedCmpOp):
     key = '>'
 
-
 @builtin
 class CmpOpGe(OrderedCmpOp):
     key = '>='
 
-
 @builtin
 class CmpOpEq(UnorderedCmpOp):
     key = '=='
 
-
 @builtin
 class CmpOpNe(UnorderedCmpOp):
     key = '!='
 
 
+class TupleCompare(AbstractTemplate):
+    def generic(self, args, kws):
+        [lhs, rhs] = args
+        tuple_types = (types.Tuple, types.UniTuple)
+        if isinstance(lhs, tuple_types) and isinstance(rhs, tuple_types):
+            for u, v in zip(lhs, rhs):
+                # Check element-wise comparability
+                res = self.context.resolve_function_type(self.key, (u, v), {})
+                if res is None:
+                    break
+            else:
+                return signature(types.boolean, lhs, rhs)
+
+@builtin
+class TupleEq(TupleCompare):
+    key = '=='
+
+@builtin
+class TupleNe(TupleCompare):
+    key = '!='
+
+@builtin
+class TupleGe(TupleCompare):
+    key = '>='
+
+@builtin
+class TupleGt(TupleCompare):
+    key = '>'
+
+@builtin
+class TupleLe(TupleCompare):
+    key = '<='
+
+@builtin
+class TupleLt(TupleCompare):
+    key = '<'
+
+
 class CmpOpIdentity(AbstractTemplate):
     def generic(self, args, kws):
         [lhs, rhs] = args
@@ -430,6 +467,17 @@ class GetItemBuffer(AbstractTemplate):
         return signature(res, ary, idx)
 
 
+@builtin
+class GetItemCPointer(AbstractTemplate):
+    key = "getitem"
+
+    def generic(self, args, kws):
+        assert not kws
+        ptr, idx = args
+        if isinstance(ptr, types.CPointer) and isinstance(idx, types.Integer):
+            return signature(ptr.dtype, ptr, normalize_index(idx))
+
+
 @builtin
 class SetItemBuffer(AbstractTemplate):
     key = "setitem"
@@ -443,6 +491,17 @@ class SetItemBuffer(AbstractTemplate):
             return signature(types.none, ary, normalize_index(idx), ary.dtype)
 
 
+@builtin
+class SetItemCPointer(AbstractTemplate):
+    key = "setitem"
+
+    def generic(self, args, kws):
+        assert not kws
+        ptr, idx, val = args
+        if isinstance(ptr, types.CPointer) and isinstance(idx, types.Integer):
+            return signature(types.none, ptr, normalize_index(idx), ptr.dtype)
+
+
 @builtin
 class Len(AbstractTemplate):
     key = types.len_type
@@ -478,6 +537,9 @@ class ArrayAttribute(AttributeTemplate):
     def resolve_flat(self, ary):
         return types.NumpyFlatType(ary)
 
+    def resolve_ctypes(self, ary):
+        return types.ArrayCTypes(ary)
+
     def generic_resolve(self, ary, attr):
         if isinstance(ary.dtype, types.Record):
             if attr in ary.dtype.fields:
@@ -485,24 +547,45 @@ class ArrayAttribute(AttributeTemplate):
                                    layout='A')
 
 
+@builtin_attr
+class ArrayCTypesAttribute(AttributeTemplate):
+    key = types.ArrayCTypes
+
+    def resolve_data(self, ctinfo):
+        return types.uintp
+
+
+
 @builtin_attr
 class NestedArrayAttribute(ArrayAttribute):
     key = types.NestedArray
 
 
+def _expand_integer(ty):
+    """
+    If *ty* is an integer, expand it to a machine int (like Numpy).
+    """
+    if isinstance(ty, types.Integer):
+        if ty.signed:
+            return max(types.intp, ty)
+        else:
+            return max(types.uintp, ty)
+    else:
+        return ty
+
 def generic_homog(self, args, kws):
     assert not args
     assert not kws
     return signature(self.this.dtype, recvr=self.this)
 
 def generic_expand(self, args, kws):
-    if isinstance(self.this.dtype, types.Integer):
-        # Expand to a machine int, not larger (like Numpy)
-        if self.this.dtype.signed:
-            return signature(max(types.intp, self.this.dtype), recvr=self.this)
-        else:
-            return signature(max(types.uintp, self.this.dtype), recvr=self.this)
-    return signature(self.this.dtype, recvr=self.this)
+    return signature(_expand_integer(self.this.dtype), recvr=self.this)
+
+def generic_expand_cumulative(self, args, kws):
+    assert isinstance(self.this, types.Array)
+    return_type = types.Array(dtype=_expand_integer(self.this.dtype),
+                              ndim=1, layout='C')
+    return signature(return_type, recvr=self.this)
 
 def generic_hetero_real(self, args, kws):
     assert not args
@@ -533,6 +616,10 @@ for fname in ["min", "max"]:
 for fname in ["sum", "prod"]:
     install_array_method(fname, generic_expand)
 
+# Functions that return a machine-width type, to avoid overflows
+for fname in ["cumsum", "cumprod"]:
+    install_array_method(fname, generic_expand_cumulative)
+
 # Functions that require integer arrays get promoted to float64 return
 for fName in ["mean", "var", "std"]:
     install_array_method(fName, generic_hetero_real)
@@ -618,116 +705,23 @@ for ty in types.number_domain:
 
 #-------------------------------------------------------------------------------
 
-@builtin_attr
-class NumbaTypesModuleAttribute(AttributeTemplate):
-    key = types.Module(types)
+def register_casters(register_global):
+    nb_types = set(types.number_domain)
+    nb_types.add(types.bool_)
 
-    def resolve_int8(self, mod):
-        return types.Function(ToInt8)
+    for restype in nb_types:
+        class Caster(AbstractTemplate):
+            key = restype
 
-    def resolve_int16(self, mod):
-        return types.Function(ToInt16)
+            def generic(self, args, kws):
+                assert not kws
+                [a] = args
+                if a in nb_types:
+                    return signature(self.key, a)
 
-    def resolve_int32(self, mod):
-        return types.Function(ToInt32)
+        register_global(restype, types.NumberClass(restype, Caster))
 
-    def resolve_int64(self, mod):
-        return types.Function(ToInt64)
-
-    def resolve_uint8(self, mod):
-        return types.Function(ToUint8)
-
-    def resolve_uint16(self, mod):
-        return types.Function(ToUint16)
-
-    def resolve_uint32(self, mod):
-        return types.Function(ToUint32)
-
-    def resolve_uint64(self, mod):
-        return types.Function(ToUint64)
-
-    def resolve_float32(self, mod):
-        return types.Function(ToFloat32)
-
-    def resolve_float64(self, mod):
-        return types.Function(ToFloat64)
-
-    def resolve_complex64(self, mod):
-        return types.Function(ToComplex64)
-
-    def resolve_complex128(self, mod):
-        return types.Function(ToComplex128)
-
-
-class Caster(AbstractTemplate):
-    def generic(self, args, kws):
-        assert not kws
-        [a] = args
-        if a in types.number_domain:
-            return signature(self.key, a)
-
-
-class ToInt8(Caster):
-    key = types.int8
-
-
-class ToInt16(Caster):
-    key = types.int16
-
-
-class ToInt32(Caster):
-    key = types.int32
-
-
-class ToInt64(Caster):
-    key = types.int64
-
-
-class ToUint8(Caster):
-    key = types.uint8
-
-
-class ToUint16(Caster):
-    key = types.uint16
-
-
-class ToUint32(Caster):
-    key = types.uint32
-
-
-class ToUint64(Caster):
-    key = types.uint64
-
-
-class ToFloat32(Caster):
-    key = types.float32
-
-
-class ToFloat64(Caster):
-    key = types.float64
-
-
-class ToComplex64(Caster):
-    key = types.complex64
-
-
-class ToComplex128(Caster):
-    key = types.complex128
-
-
-builtin_global(types, types.Module(types))
-builtin_global(types.int8, types.Function(ToInt8))
-builtin_global(types.int16, types.Function(ToInt16))
-builtin_global(types.int32, types.Function(ToInt32))
-builtin_global(types.int64, types.Function(ToInt64))
-builtin_global(types.uint8, types.Function(ToUint8))
-builtin_global(types.uint16, types.Function(ToUint16))
-builtin_global(types.uint32, types.Function(ToUint32))
-builtin_global(types.uint64, types.Function(ToUint64))
-builtin_global(types.float32, types.Function(ToFloat32))
-builtin_global(types.float64, types.Function(ToFloat64))
-builtin_global(types.complex64, types.Function(ToComplex64))
-builtin_global(types.complex128, types.Function(ToComplex128))
+register_casters(builtin_global)
 
 #------------------------------------------------------------------------------
 
