@@ -9,10 +9,6 @@ import threading
 import timeit
 import math
 import sys
-try:
-    import builtins
-except ImportError:
-    import __builtin__ as builtins
 
 import numpy
 
@@ -23,11 +19,13 @@ from numba.config import PYVERSION, MACHINE_BITS
 IS_PY3 = PYVERSION >= (3, 0)
 
 if IS_PY3:
+    import builtins
     INT_TYPES = (int,)
     longint = int
     get_ident = threading.get_ident
 else:
     import thread
+    import __builtin__ as builtins
     INT_TYPES = (int, long)
     longint = long
     get_ident = thread.get_ident
@@ -140,7 +138,8 @@ class SortedSet(collections.Set):
 
 class UniqueDict(dict):
     def __setitem__(self, key, value):
-        assert key not in self
+        if key in self:
+            raise AssertionError("key already in dictionary: %r" % (key,))
         super(UniqueDict, self).__setitem__(key, value)
 
 
@@ -269,6 +268,11 @@ def benchmark(func, maxsec=1):
 RANGE_ITER_OBJECTS = (builtins.range,)
 if PYVERSION < (3, 0):
     RANGE_ITER_OBJECTS += (builtins.xrange,)
+    try:
+        from future.types.newrange import newrange
+        RANGE_ITER_OBJECTS += (newrange,)
+    except ImportError:
+        pass
 
 
 # Backported from Python 3.4: functools.total_ordering()
